/**
 * WlanMap 0.9
 *
 * Copyright 2013      Konrad Enzensberger <e.konrad@mpegcode.com>
 * All rights reserved.
 *
 * WlanMap is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * WlanMap is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.*
 *
 * You should have received a copy of the GNU General Public License
 * along with wlanapi library.  If not, see <http://www.gnu.org/licenses/>.
 */

/* INCLUDE FILES **************************************************************/


#include <QtGui>

#include "mainwindow.h"
#include "wlandef.h"

#include "MarbleDirs.h"
#include "MarbleAboutDialog.h"
#include "SunControlWidget.h"
#include "MarbleLocale.h"
#include "AbstractDataPlugin.h"
#include "AbstractFloatItem.h"
#include "HttpDownloadManager.h"
#include "MarbleMap.h"
#include "MarbleModel.h"
#include "ViewportParams.h"
#include "PluginManager.h"
#include "RoutingRunnerPlugin.h"
#include "ElevationRunnerPlugin.h"
#include "planet.h"

#include "SearchInputWidget.h"
#include "MapWizard.h"
#include "MapViewSettingsDialog.h"
#include "GeoDataDocument.h"
#include "GeoDataGroundOverlay.h"
#include "GeoDataTreeModel.h"
#include "PositionTracking.h"

#include "uic_WlanSettings.h"

#include "..\wlanapi\wlanapi.h"

using namespace Marble;

namespace
{
    const char* POSITION_STRING = "Position:";
    const char* DISTANCE_STRING = "Altitude:";
	const char* ELEVATION_STRING = "Elevation:";
}

MainWindow::MainWindow()
{
	setAttribute(Qt::WA_DeleteOnClose);

	m_dCenterLon=11.0;
	m_dCenterLat=47.5;
	m_iZoom=2000;
	m_iProjection=0;

	m_bGPS=false;
	m_dGpsLon=0;
	m_dGpsLat=0;

	m_lOverlay=0;
	m_lLayer=0;
	m_lActiveLayer=0;
	m_lAdapterOverlay=0;
	m_lShadingOverlay=0;
	m_lShadingLayer=0;
	m_iCount=0;

	m_bContinous=true;
	m_bMP=false;//default: manually measure point setting
	m_iAP=0;
	m_iTimer=5;//s

	fileMenu=NULL;
    wlanMenu=NULL;
    helpMenu=NULL;
	mapMenu=NULL;
	settingsMenu=NULL;

    fileToolBar=NULL;
    wlanToolBar=NULL;

    openAct=NULL;
    saveAct=NULL;
    saveAsAct=NULL;
    exitAct=NULL;

	wlanDetectAct=NULL;
	wlanMeasureAct=NULL;

	settingsAct=NULL;

	basemapMenu=NULL;
	osmAct=NULL;
	googleAct=NULL;

	globeAct=NULL;
	mercatorAct=NULL;
	flatAct=NULL;
	importAct=NULL;

    aboutAct=NULL;
    aboutQtAct=NULL;
	aboutMarbleAct=NULL;
	manualAct=NULL;

	m_positionLabel=NULL;
	m_distanceLabel=NULL;   
	m_elevationLabel=NULL;
	m_infoLabel=NULL;

	m_controlView=NULL;

	QCoreApplication::setOrganizationDomain("WalnMapDomain");
    QCoreApplication::setOrganizationName("emd");
    QCoreApplication::setApplicationName("WlanMap");
    QCoreApplication::setApplicationVersion("0.9");
}

MainWindow::~MainWindow()
{
	//free google satellite cache
	QString mapdir=m_qsMapDataPath;
	mapdir+="/maps/earth/googlesat"; 
	deleteMapDirectory(mapdir);

	QApMp* pMP=NULL;
	if(m_qhMP.size()>0)
	{
		QHash <long,QApMp*>::const_iterator iter = m_qhMP.constBegin();
		for ( ; iter != m_qhMP.constEnd(); ++iter ) 
		{
			pMP = iter.value();
			if(pMP!=NULL)
				delete pMP;
			pMP=NULL;
		}
	}
	m_qhMP.clear();

	writeSettings();

	if(m_controlView!=NULL)
		delete m_controlView;
}

void MainWindow::closeEvent(QCloseEvent *event)
{
  
}

void MainWindow::open()
{
    QStringList fileNames;

	fileNames = QFileDialog::getOpenFileNames(this, tr("Open Geo layer"),
                            m_lastFileOpenPath,
                            tr("All Supported Files (*.kml);;Google Earth KML (*.km*)"));
	
	if ( !fileNames.isEmpty() ) {
        const QString firstFile = fileNames.first();
        m_lastFileOpenPath = QFileInfo( firstFile ).absolutePath();
    }

	QString ext;
	foreach( const QString &fileName, fileNames ) 
	{
		ext = fileName.right(3);
	
		m_controlView->marbleModel()->addGeoDataFile( fileName );
    }
}

bool MainWindow::save()
{ 
	MarbleWidget* pWidget = m_controlView->marbleWidget();
	if(pWidget==NULL)return false;

	TreeViewWidget* pTree = pWidget->tree();
	if(pTree==NULL)return false;

	QString filename = QFileDialog::getSaveFileName( 0, tr("Measurement file name"),
                            m_lastFileOpenPath,
                            tr("All Supported Files (*.kml);;Google Kml (*.kml)"));

	if(!filename.isEmpty())
	{
		m_lastFileOpenPath = QFileInfo( filename ).absolutePath();
		pTree->exportToKml(m_lOverlay,filename);
	}

   return true;
}

bool MainWindow::saveAs()
{
	MarbleWidget* pWidget = m_controlView->marbleWidget();
	if(pWidget==NULL)return false;

	TreeViewWidget* pTree = pWidget->tree();
	if(pTree==NULL)return false;

	QString filename = QFileDialog::getSaveFileName( 0, tr("Measurement file name"),
                            m_lastFileOpenPath,
                            tr("All Supported Files (*.kml);;Google Kml (*.kml)"));

	if(!filename.isEmpty())
	{
		m_lastFileOpenPath = QFileInfo( filename ).absolutePath();
		pTree->exportToKml(m_lOverlay,filename);
	}

	return true;   
}

void MainWindow::about()
{
    QMessageBox::about(this, tr("About WlanMap"),
             tr("<p><b>WlanMap</b> "
			    "<p>Version 0.9 "
				"<p>Copyright <a href='http://www.mpegcode.com'>emd</a> @ 2013 "
				"<p>License <a href='http://www.gnu.org/licenses/gpl-3.0'>GNU GPL "
				"<p>Some icons of this <a href='http://openiconlibrary.sourceforge.net/downloads.html'>GPL package</a> are used "
				"<p>WlanMap uses a modified version of: <a href='https://github.com/maurice2k/wlanapi'>wlanapi</a></p>"));
}

void MainWindow::aboutMarblewidget()
{
	MarbleWidget* pWidget = m_controlView->marbleWidget();
	if(pWidget==NULL)return;

	MarbleAboutDialog dlg(this);
    dlg.setApplicationTitle( tr( "WlanMap %1" ).arg( ControlView::applicationVersion() ) );
    dlg.exec();
}

void MainWindow::init()
{
	readSettings();
	
	if(m_qsMapDataPath.isEmpty())
		m_qsMapDataPath="D:/mapdata";

	bool bMapDataExist=true;
	bool bMapDataEmpty=true;
	QDir dir1(m_qsMapDataPath);
	QString mappath = m_qsMapDataPath + '/' + "maps" + '/' + "earth" + '/' + "openstreetmap";
	QDir dir2(mappath);

	QString msg;
	if(!dir1.exists())
	{
		bMapDataExist=false;
		msg=tr("mapdata path: \"<b>%1</b>\"not found.<br>Do you like to select a new mapdata folder?");
	}
	else
	{
		if(!dir2.exists())
			bMapDataEmpty=false;
		
		msg=tr("No maps are found in: \"<b>%1</b>\"<br>Do you like to select a new mapdata folder?");

	}
	msg = msg.arg(m_qsMapDataPath);

	if(!bMapDataExist || !bMapDataEmpty)
	{
		QMessageBox::StandardButton ret = QMessageBox::warning(this, QWidget::tr("WlanMap"),msg,QMessageBox::Yes|QMessageBox::No);

		if(ret==QMessageBox::Yes)
		{
			QFileDialog qFlDlg;
			msg="C:/";
			QString qsDir(m_qsMapDataPath);
			QString qsCaption(tr("Select a new mapdata folder"));

			m_qsMapDataPath = qFlDlg.getExistingDirectory(NULL,qsCaption,msg,QFileDialog::ShowDirsOnly);
		}
	}

	MarbleDirs::setMarbleDataPath(m_qsMapDataPath);
	QString qsPlugin = QApplication::applicationDirPath();
	qsPlugin += "/plugins/wlanmap/";
	MarbleDirs::setMarblePluginPath(qsPlugin);

	m_controlView = new ControlView( this);
	if(m_controlView==NULL)return;

	setWindowTitle( tr("WlanMap") );
    setWindowIcon( QIcon(":/images/gps32.png") );
    setCentralWidget( m_controlView );

  //  textEdit = new QTextEdit;
 //   setCentralWidget(textEdit);

    createActions();
    createMenus();
    createToolBars();
    createStatusBar();

	MarbleWidget* pWidget = m_controlView->marbleWidget();
	if(pWidget==NULL)return;

	if(m_qsThemeId.isEmpty())
		m_qsThemeId = "earth/openstreetmap/openstreetmap.dgml";;//"earth/openstreetmap/openstreetmap.dgml";
	
	QString qsFullPath = m_qsMapDataPath; 
	qsFullPath += "/";
	qsFullPath += m_qsThemeId; 

	Marble::Projection prj = (Marble::Projection)m_iProjection;//Marble::Projection::Spherical;
	pWidget->setMapThemeId(m_qsThemeId);

	// Set home position
    m_controlView->marbleModel()->setHome(m_dCenterLon,m_dCenterLat,m_iZoom);
	pWidget->centerOn(m_dCenterLon,m_dCenterLat);
	
	pWidget->setProjection(prj);

	//without we have huge cpu usage while dimming
	pWidget->setShowSunShading(false);
	pWidget->setShowCityLights(false);
	pWidget->setShowSunInZenith(false);

	m_controlView->marbleModel()->setWorkOffline(false);

	bool bConnect = connect( pWidget->model()->positionTracking(),
             SIGNAL( gpsLocation( GeoDataCoordinates, qreal ) ),
             this, SLOT( receiveGpsCoordinates( GeoDataCoordinates, qreal ) ) );

	bConnect = connect( pWidget->model(),
             SIGNAL( homeChanged( GeoDataCoordinates ) ),
             this, SLOT( slotHomeChanged( GeoDataCoordinates) ) );

	TreeViewWidget* pTree = pWidget->tree();
	if(pTree!=NULL)
	{
		connect( pTree,
             SIGNAL( signalSendEvent(long,long,void*,long) ),
             this, SLOT( slotSendEvent(long,long,void*,long) ) );
	}

	setupStatusBar();
	showWlanSettings();
    setUnifiedTitleAndToolBarOnMac(true);
}

void MainWindow::createActions()
{
    openAct = new QAction(QIcon(":/images/open.png"), tr("&Open..."), this);
    openAct->setShortcuts(QKeySequence::Open);
    openAct->setStatusTip(tr("Open Google kml file"));
    connect(openAct, SIGNAL(triggered()), this, SLOT(open()));

    saveAct = new QAction(QIcon(":/images/save.png"), tr("&Save"), this);
    saveAct->setShortcuts(QKeySequence::Save);
    saveAct->setStatusTip(tr("Save complete measurement as Google kml"));
    connect(saveAct, SIGNAL(triggered()), this, SLOT(save()));

    saveAsAct = new QAction(tr("Save &As..."), this);
    saveAsAct->setShortcuts(QKeySequence::SaveAs);
    saveAsAct->setStatusTip(tr("Save complete measurement as Google kml under a new name"));
    connect(saveAsAct, SIGNAL(triggered()), this, SLOT(saveAs()));

    exitAct = new QAction(tr("E&xit"), this);
    exitAct->setShortcuts(QKeySequence::Quit);
    exitAct->setStatusTip(tr("Exit the application"));
    connect(exitAct, SIGNAL(triggered()), qApp, SLOT(closeAllWindows()));

    wlanDetectAct = new QAction(QIcon(":/images/ap32.png"), tr("W&lan detection"), this);
 //   wlanDetectAct->setShortcuts(QKeySequence::Cut);
    wlanDetectAct->setStatusTip(tr("Start WLAN access point detection "));
    connect(wlanDetectAct, SIGNAL(triggered()), this, SLOT(slotWlanDetection()));

	wlanMeasureAct = new QAction(QIcon(":/images/signal_32.png"), tr("Wlan s&ignal"), this);
 //   wlanMeasureAct->setShortcuts(QKeySequence::Cut);
    wlanMeasureAct->setStatusTip(tr("WLAN signal measurement "));
	wlanMeasureAct->setCheckable(true);
	wlanMeasureAct->setEnabled(false);
	
    connect(wlanMeasureAct, SIGNAL(triggered()), this, SLOT(slotWlanMeasure()));

	heatmapAct = new QAction(QIcon(":/images/heatmap.png"), tr("&Heatmap"), this);
 //   heatmapAct->setShortcuts(QKeySequence::Cut);
    heatmapAct->setStatusTip(tr("Create heatmap from measure points"));
    connect(heatmapAct, SIGNAL(triggered()), this, SLOT(slotCreateHeatmap()));

	settingsAct = new QAction(QIcon(":/images/tools_32.png"), tr("&Settings"), this);
    //wlanDetectAct->setShortcuts(QKeySequence::Cut);
    settingsAct->setStatusTip(tr("WLAN measurement settings "));
    connect(settingsAct, SIGNAL(triggered()), this, SLOT(slotSettings()));

	//basemap menu
	basemapMenu = new QMenu(tr("&Load Map"));

	osmAct = new QAction(QIcon(":/images/basemap.png"), tr("&openstreetmap"), this);
    //wlanDetectAct->setShortcuts(QKeySequence::Cut);
	osmAct->setCheckable(true);
	osmAct->setChecked(true);
    osmAct->setStatusTip(tr("Load openstreetmap "));
    connect(osmAct, SIGNAL(triggered()), this, SLOT(slotLoadOsmMap()));
	basemapMenu->addAction(osmAct);

	googleAct = new QAction(QIcon(":/images/basemap.png"), tr("&Google satellite"), this);
    //wlanDetectAct->setShortcuts(QKeySequence::Cut);
	googleAct->setCheckable(true);
	googleAct->setChecked(false);
    googleAct->setStatusTip(tr("Load Google satellite "));
    connect(googleAct, SIGNAL(triggered()), this, SLOT(slotLoadGoogleMap()));
	basemapMenu->addAction(googleAct);

	//Projection menu
	projMenu = new QMenu(tr("Map Projection"));

	globeAct = new QAction(QIcon(":/images/map-globe.png"),tr("Globe"), this);
	globeAct->setCheckable(true);
	globeAct->setChecked(true);
	connect(globeAct, SIGNAL(triggered()), this, SLOT(SlotSetGlobeProj()));
	projMenu->addAction(globeAct);

	mercatorAct = new QAction(QIcon(":/images/map-mercator.png"),tr("Mercator"), this);
	mercatorAct->setCheckable(true);
	connect(mercatorAct, SIGNAL(triggered()), this, SLOT(SlotSetMercProj()));
	projMenu->addAction(mercatorAct);

	flatAct = new QAction(QIcon(":/images/map-flat.png"),tr("Flat Map"), this);
	flatAct->setCheckable(true);
	connect(flatAct, SIGNAL(triggered()), this, SLOT(SlotSetEquiProj()));
	projMenu->addAction(flatAct);

	importAct = new QAction(QIcon(":/images/import.png"),tr("Import raster map"), this);
	importAct->setStatusTip(tr("Import georeferenced raster images"));
	connect(importAct, SIGNAL(triggered()), this, SLOT(SlotAddImageLayer()));

    aboutAct = new QAction(tr("&About WlanMap"), this);
    aboutAct->setStatusTip(tr("Show the application's About box"));
    connect(aboutAct, SIGNAL(triggered()), this, SLOT(about()));

    aboutQtAct = new QAction(tr("About &Qt"), this);
    aboutQtAct->setStatusTip(tr("Show the Qt library's About box"));
    connect(aboutQtAct, SIGNAL(triggered()), qApp, SLOT(aboutQt()));

	aboutMarbleAct = new QAction(tr("About &marblewidget"), this);
    aboutMarbleAct->setStatusTip(tr("Show the marblewidget library's About box"));
    connect(aboutMarbleAct, SIGNAL(triggered()), this, SLOT(aboutMarblewidget()));

	manualAct = new QAction(tr("&UserManual"), this);
    manualAct->setStatusTip(tr("Open the UserManual"));
    connect(manualAct, SIGNAL(triggered()), this, SLOT(help()));

}

//! [implicit tr context]
void MainWindow::createMenus()
{
    fileMenu = menuBar()->addMenu(tr("&File"));
    fileMenu->addAction(openAct);
    fileMenu->addAction(saveAct);
    fileMenu->addAction(saveAsAct);
    fileMenu->addSeparator();
    fileMenu->addAction(exitAct);

    wlanMenu = menuBar()->addMenu(tr("&Wlan"));
    wlanMenu->addAction(wlanDetectAct);
    wlanMenu->addAction(wlanMeasureAct);
//	wlanMenu->addAction(gpsAct);

	settingsMenu = menuBar()->addMenu(tr("&Settings"));
	settingsMenu->addAction(settingsAct);

//	wlanMeasureAct->setMenu(settingsMenu);
	
	mapMenu = menuBar()->addMenu(tr("&Map"));
	mapMenu->addMenu(basemapMenu);
	mapMenu->addMenu(projMenu);
	mapMenu->addAction(importAct);

    helpMenu = menuBar()->addMenu(tr("&Help"));
    helpMenu->addAction(aboutAct);
    helpMenu->addAction(aboutQtAct);
	helpMenu->addAction(aboutMarbleAct);
	helpMenu->addAction(manualAct);
}

void MainWindow::createToolBars()
{

    fileToolBar = addToolBar(tr("File"));
    fileToolBar->addAction(openAct);
    fileToolBar->addAction(saveAct);

    wlanToolBar = addToolBar(tr("Wlan"));
	wlanToolBar->addAction(settingsAct);
	wlanToolBar->addSeparator();
    wlanToolBar->addAction(wlanDetectAct);
    wlanToolBar->addAction(wlanMeasureAct);
	wlanToolBar->addSeparator();
	wlanToolBar->addAction(heatmapAct);
}

void MainWindow::createStatusBar()
{
    statusBar()->showMessage(tr("Ready"));
}

void MainWindow::readSettings()
{
    QSettings settings;
    QPoint pos = settings.value("pos", QPoint(200, 200)).toPoint();
    QSize size = settings.value("size", QSize(400, 400)).toSize();
    move(pos);
    resize(size);

	QVariant var;
	var = settings.value("ScanMode",false);
	m_bContinous = var.toBool();
	var = settings.value("AccesspointMode",0);
	m_iAP = var.toInt();
	var = settings.value("MeasurePoint",false);
	m_bMP = var.toBool();
	var = settings.value("MeasureTimer",5);
	m_iTimer = var.toInt();

	var = settings.value("ThemeID","earth/openstreetmap/openstreetmap.dgml");
	m_qsThemeId = var.toString();
	var = settings.value("Longitude",11.0);
	m_dCenterLon = var.toDouble();
	var = settings.value("Latitude",47.5);
	m_dCenterLat = var.toDouble();
	var = settings.value("Projection",0);
	m_iProjection = var.toInt();
	var = settings.value("Mapdata","C:/mapdata");
	m_qsMapDataPath = var.toString();
	var = settings.value("Path",QString());
	m_lastFileOpenPath = var.toString();
	var = settings.value("Zoom",2500);
	m_iZoom = var.toInt();
}

void MainWindow::writeSettings()
{
    QSettings settings;
    settings.setValue("pos", pos());
    settings.setValue("size", size());

	settings.setValue("ScanMode",m_bContinous);
	settings.setValue("AccesspointMode",m_iAP);
	settings.setValue("MeasurePoint",m_bMP);
	settings.setValue("MeasureTimer",m_iTimer);

	MarbleWidget* pWidget = m_controlView->marbleWidget();
	if(pWidget==NULL)return;

	m_dCenterLon = pWidget->centerLongitude();
	m_dCenterLat = pWidget->centerLatitude();
	m_iProjection = pWidget->projection();
	m_iZoom = pWidget->zoom();

	settings.setValue("ThemeID",m_qsThemeId);
	settings.setValue("Longitude",m_dCenterLon);
	settings.setValue("Latitude",m_dCenterLat);
	settings.setValue("Projection",m_iProjection);
	settings.setValue("Mapdata",m_qsMapDataPath);
	settings.setValue("Path",m_lastFileOpenPath);
	settings.setValue("Zoom",m_iZoom);
}

void MainWindow::help()
{
	QString qsFile("file:///");
	QString qsPath("/UserManual/WlanMap.pdf");
	QString qsUrl = qsFile + QApplication::applicationDirPath() + qsPath;
	QDesktopServices::openUrl(QUrl(qsUrl));
}

void MainWindow::slotLoadOsmMap()
{
	MarbleWidget* pWidget = m_controlView->marbleWidget();
	if(pWidget==NULL)return;

	if(m_qsThemeId == "earth/openstreetmap/openstreetmap.dgml")
		return;
		
	googleAct->setChecked(false);
	osmAct->setChecked(true);
	m_qsThemeId = "earth/openstreetmap/openstreetmap.dgml";

	pWidget->setMapThemeId(m_qsThemeId);
}

void MainWindow::slotLoadGoogleMap()
{
	MarbleWidget* pWidget = m_controlView->marbleWidget();
	if(pWidget==NULL)return;

	if(m_qsThemeId == "earth/googlesat/google.dgml")
		return;
		
	googleAct->setChecked(true);
	osmAct->setChecked(false);
	m_qsThemeId = "earth/googlesat/googlesat.dgml";

	pWidget->setMapThemeId(m_qsThemeId);

}
	
void MainWindow::SlotSetGlobeProj()
{
	globeAct->setChecked(true);
	mercatorAct->setChecked(false);
	flatAct->setChecked(false);
	setProjection(0);
}
	
void MainWindow::SlotSetMercProj()
{
	globeAct->setChecked(false);
	mercatorAct->setChecked(true);
	flatAct->setChecked(false);
	setProjection(2);
}
	
void MainWindow::SlotSetEquiProj()
{
	globeAct->setChecked(false);
	mercatorAct->setChecked(false);
	flatAct->setChecked(true);
	setProjection(1);
}

void MainWindow::SlotAddImageLayer()
{
	QStringList fileNames = QFileDialog::getOpenFileNames(this, tr("Open image file"),
                            m_lastFileOpenPath,
                            tr("All Supported Files (*.tif *.jpg *.png *.bmp *.tab);;GeoTiff  (*.tif);;MapInfo  (*.tab)"));

	if ( !fileNames.isEmpty() ) {
        const QString firstFile = fileNames.first();
        m_lastFileOpenPath = QFileInfo( firstFile ).absolutePath();
    }

	foreach( const QString &fileName, fileNames ) {
         m_controlView->marbleModel()->addGeoDataFile( fileName );
    }
}

void MainWindow::slotWlanDetection()
{
	MarbleWidget* pWidget = m_controlView->marbleWidget();
	if(pWidget==NULL)return;

	TreeViewWidget* pTree = pWidget->tree();
	if(pTree==NULL)return;

	wlanMeasureAct->setEnabled(false);
	m_bGPS=false;
	m_iCount=0;

	if(m_lOverlay!=0 || m_lAdapterOverlay!=0)
	{
		QMessageBox::StandardButton reply;
		reply = QMessageBox::question(this, "WlanMap", "Delete all scans?",QMessageBox::Yes|QMessageBox::No);
		if (reply == QMessageBox::Yes) 
		{
			if(m_lAdapterOverlay!=0)
				pTree->slotDeleteObject(m_lAdapterOverlay,MST_DELETE_OVERLAY);
			if(m_lOverlay!=0)
				pTree->slotDeleteObject(m_lOverlay,MST_DELETE_OVERLAY);

			qlAdapter.clear();
			wlanlay.clear();
			m_qhMP.clear();

			if(m_lShadingOverlay!=0)
			{
				QApMp* pMP=NULL;
				if(m_qhMP.size()>0)
				{
					QHash <long,QApMp*>::const_iterator iter = m_qhMP.constBegin();
					for ( ; iter != m_qhMP.constEnd(); ++iter ) 
					{
						pMP = iter.value(); 
						if(pMP!=NULL)
							delete pMP;
						pMP=NULL;
					}
				}
				m_qhMP.clear();
				shadinglay.clear();
				if(m_lShadingOverlay!=0)
					pTree->slotDeleteObject(m_lShadingOverlay,MST_DELETE_OVERLAY);

				m_lOverlay=0;
				m_lAdapterOverlay=0;
				m_lShadingOverlay=0;
				m_lShadingLayer=0;
				m_lActiveLayer=0;
				m_lLayer=0;
				m_iCount=0;
			}
		}
	}

	double lat=0;
	double lon=0;
	char quality[64];
	short ccode;

	wlanapi wi;// = new wlanapi();
	const ADAPTER_LIST &adapter_list = wi.get_adapter_list();
    const AP_LIST &ap_list = wi.get_ap_list(NULL);

	QStringList qsl;
//	long overlayID =0;
	long uLayerID = 0;
	long uGroupID = 0;
	QString adapter;// = QString::fromWCharArray((const wchar_t*)adapter_list.);
	QString tmp("Adapter");
	BSTR bstr;

	if(!qlAdapter.contains(tmp))
	{
		qsl << "-399" << "-399" << tmp;
		pTree->slotAddObject(qsl,0,MST_ADD_OVERLAY,m_lAdapterOverlay);
		qlAdapter.insert(tmp,m_lAdapterOverlay);
	}

	for (ADAPTER_LIST::const_iterator it = adapter_list.begin(); it < adapter_list.end(); ++it) 
	{
		adapter = QString::fromWCharArray((const wchar_t*)it->description);
		if(!qlAdapter.contains(adapter))
		{
			qsl.clear();
			qsl << "-399" << "-399" << adapter;
			pTree->slotAddObject(qsl,m_lAdapterOverlay,MST_ADD_LAYER,uLayerID);

			//set properties
			tmp.clear();
			tmp="IGNORE:CACHE=TRUE;DELETE=FALSE;EDIT=TRUE;O_DELETE=TRUE;O_EDIT=TRUE;FLAG=3;";
			pTree->slotSetObjectProperties(tmp,uLayerID,161);

			qlAdapter.insert(adapter,uLayerID);
			
		}
	}
	qsl.clear();
	tmp = "Accesspoints";
	if(!qlAdapter.contains(tmp))
	{
		qsl << "-399" << "-399" << tmp;
		pTree->slotAddObject(qsl,0,MST_ADD_OVERLAY,m_lOverlay);
		qlAdapter.insert(tmp,m_lOverlay);		
	}

	int iC = ap_list.size();
	int iSig = 0;
	int iRSSI= 0;
	//for(int i=0;i<iC;i++)
	for (AP_LIST::const_iterator it = ap_list.begin(); it < ap_list.end(); ++it)
	{
		QString wlanx = QString::fromUtf8((const char*)it->name);
		QString mac("\nMAC: %1-%2-%3-%4-%5-%6");
		mac = mac.arg(it->mac_address.u[0]).arg(it->mac_address.u[1]).arg(it->mac_address.u[2]).arg(it->mac_address.u[3]).arg(it->mac_address.u[4]).arg(it->mac_address.u[5]);
		//unique access point key
		wlanx += mac;
		iRSSI = it->rssi;
		if (iRSSI == -100)
             iSig = 0;
         else if (iRSSI == -50)   
             iSig = 100;
         else
             iSig = (iRSSI+100)*2;

		 if(wlanx.isEmpty())
			 continue;

		if(!wlanlay.contains(wlanx))
		{
			//add layer
			qsl.clear();
			qsl << "-399" << "-399" << wlanx;
			pTree->slotAddObject(qsl,m_lOverlay,MST_ADD_LAYER,m_lLayer);

			//set properties
			tmp.clear();
			tmp="IGNORE:CACHE=TRUE;DELETE=FALSE;EDIT=TRUE;O_DELETE=TRUE;O_EDIT=TRUE;FLAG=3;";
			pTree->slotSetObjectProperties(tmp,m_lLayer,161);

			pTree->slotLayerVisible(m_lLayer,wlanx,false);

			wlanlay.insert(wlanx,m_lLayer);
		}
/*		else
		{
			uLayerID = wlanlay.value(wlanx);

			//next group
			tmp = "SignalQuality (%1)";
			tmp = tmp.arg(iSig);
			qsl.clear();
			qsl << "-399" << "-399" << tmp;
			pTree->slotAddObject(qsl,uLayerID,MST_ADD_GROUP,uGroupID);

			//next group
			tmp = "RSSI (%1 dBm)";
			tmp = tmp.arg(iRSSI);
			qsl.clear();
			qsl << "-399" << "-399" << tmp;
			pTree->slotAddObject(qsl,uLayerID,MST_ADD_GROUP,uGroupID);

		}*/
	}

	if(!m_bMP && m_iAP>0 && wlanlay.size()>0)
		wlanMeasureAct->setEnabled(true);
}

void MainWindow::slotWlanMeasure()
{
	switch(m_iAP)
	{
		case 0:
			scanSelectedAP();
			break;
		case 1:
			scanStrongestAP();
			break;
		case 2:
			scanAllAP();
			break;
		default:
			scanSelectedAP();
			break;
	}

//	if(m_iAP==0 && m_iCount==10)
//		createHeatmap();
}
void MainWindow::scanSelectedAP()
{
	MarbleWidget* pWidget = m_controlView->marbleWidget();
	if(pWidget==NULL)return;

	TreeViewWidget* pTree = pWidget->tree();
	if(pTree==NULL)return;

	if(wlanMeasureAct->isChecked()&&m_bContinous)
		 QTimer::singleShot( m_iTimer*1000, this, SLOT( slotWlanMeasure() ) );
//	if(wlanMeasureAct->isChecked()&&!m_bContinous)
//		wlanMeasureAct->setChecked(false);

	if(!m_bGPS)return;//no position set

	wlanapi wi;// = new wlanapi();
	const ADAPTER_LIST &adapter_list = wi.get_adapter_list();
    const AP_LIST &ap_list = wi.get_ap_list(NULL);

	QStringList qsl;
	long overlayID =0;
	long uLayerID = 0;
	long uGroupID = 0;
	long objectID=0;
	double lat=0;
	double lon=0;
	char quality[64];
	short ccode=0;

	QString tmp;
	QString qsMP;
	QString path = QApplication::applicationDirPath();
	QString red("%1/red.png");
	QString orange("%1/orange.png");
	QString oryel("%1/oryel.png");
	QString yellow("%1/yellow.png");
	QString lblue("%1/lightblue.png");

	red = red.arg(path);
	orange = orange.arg(path);
	oryel = oryel.arg(path);
	yellow = yellow.arg(path);
	lblue = lblue.arg(path);
	
	//AP info
	int iC = ap_list.size();
	int iSig = 0;
	int iRSSI= 0;
	int iLinkQuality=0;
	double iChannelFreq=0;
	int iMaxDataTransRate=0;

	for (AP_LIST::const_iterator it = ap_list.begin(); it < ap_list.end(); ++it)
	{
		QString ssid = QString::fromUtf8((const char*)it->name);
		QString mac("\nMAC: %1-%2-%3-%4-%5-%6");
		mac = mac.arg(it->mac_address.u[0]).arg(it->mac_address.u[1]).arg(it->mac_address.u[2]).arg(it->mac_address.u[3]).arg(it->mac_address.u[4]).arg(it->mac_address.u[5]);
		//unique access point key
		QString wlanx = ssid + mac;

		iRSSI = it->rssi;
		if (iRSSI == -100)
             iSig = 0;
         else if (iRSSI == -50)   
             iSig = 100;
         else
             iSig = (iRSSI+100)*2; 

		 if(iSig>100)iSig=100;

		iLinkQuality=it->linkQuality;
		iChannelFreq=(double)it->uChannelFreq/1000000.0;//GHz
		iMaxDataTransRate=it->usRateSet;//Mbps

		 if(wlanx.isEmpty())
			 continue;

		 uLayerID = wlanlay.value(wlanx);
		 if(uLayerID==m_lActiveLayer)
		 {
			  //next group
			tmp = "MP-%1";
			tmp = tmp.arg(m_iCount);
			qsl.clear();
			qsl << "-399" << "-399" << tmp;
			pTree->slotAddObject(qsl,uLayerID,MST_ADD_GROUP,uGroupID);
			pTree->slotMoveObject(uGroupID,m_dGpsLon,m_dGpsLat);
			m_iCount++;

			QString qsTool = "SSID: %1;Signal: %2;RSSI: %3 dBm;LinkQuality: %4;ChannelFrequency: %5 GHz;TransferRate max: %6 Mbps";
			qsTool = qsTool.arg(ssid).arg(iSig).arg(iRSSI).arg(iLinkQuality).arg(iChannelFreq).arg(iMaxDataTransRate);

			int iPos=0;
			QList<QApMp*> pAllMP = m_qhMP.values(uLayerID);
			iPos = pAllMP.size();
			QApMp* pMP = new QApMp(uGroupID,m_dGpsLon,m_dGpsLat,iSig,iRSSI,iLinkQuality,iPos);
			m_qhMP.insertMulti(uLayerID,pMP);//add measure point

			//icon
			if(iSig>=70)
				qsMP = red;// + ";" + tmp;
			else if(iSig>=52)
				qsMP = orange;// + ";" + tmp;
			else if(iSig>=40)
				qsMP = oryel;
			else if(iSig>=28)
				qsMP = yellow;
			else
				qsMP = lblue;// + ";" + tmp;

			pTree->slotAddImageObject(qsMP,uGroupID,MST_ADD_ICON,0,0,0,-1,objectID,false);
			pTree->slotAddDescription(qsTool,uGroupID,111,1,0,objectID);//ToolTip
			objectID=0;
			tmp="Sig: %1 / RSSI: %2 dBm";
			tmp=tmp.arg(iSig).arg(iRSSI);
			pTree->slotAddDescription(tmp,uGroupID,0,0,0,objectID);//Text tree entry
			pTree->slotObjectVisible(objectID,false);
			
		}

	}
}

void MainWindow::scanAllAP()
{
	MarbleWidget* pWidget = m_controlView->marbleWidget();
	if(pWidget==NULL)return;

	TreeViewWidget* pTree = pWidget->tree();
	if(pTree==NULL)return;

	if(wlanMeasureAct->isChecked()&&m_bContinous)
		 QTimer::singleShot( m_iTimer*1000, this, SLOT( slotWlanMeasure() ) );
//	if(wlanMeasureAct->isChecked()&&!m_bContinous)
//		wlanMeasureAct->setChecked(false);

	if(!m_bGPS)return;//no position set

	wlanapi wi;// = new wlanapi();
	const ADAPTER_LIST &adapter_list = wi.get_adapter_list();
    const AP_LIST &ap_list = wi.get_ap_list(NULL);

	QStringList qsl;
	long overlayID =0;
	long uLayerID = 0;
	long uGroupID = 0;
	long objectID=0;
	double lat=0;
	double lon=0;
	char quality[64];
	short ccode=0;

	QString tmp;
	QString qsMP;
	QString path = QApplication::applicationDirPath();
	QString red("%1/red.png");
	QString orange("%1/orange.png");
	QString oryel("%1/oryel.png");
	QString yellow("%1/yellow.png");
	QString lblue("%1/lightblue.png");

	red = red.arg(path);
	orange = orange.arg(path);
	oryel = oryel.arg(path);
	yellow = yellow.arg(path);
	lblue = lblue.arg(path);
	
	//AP info
	int iC = ap_list.size();
	int iSig = 0;
	int iRSSI= 0;
	int iLinkQuality=0;
	double iChannelFreq=0;
	int iMaxDataTransRate=0;

	for (AP_LIST::const_iterator it = ap_list.begin(); it < ap_list.end(); ++it)
	{
		QString ssid = QString::fromUtf8((const char*)it->name);
		QString mac("\nMAC: %1-%2-%3-%4-%5-%6");
		mac = mac.arg(it->mac_address.u[0]).arg(it->mac_address.u[1]).arg(it->mac_address.u[2]).arg(it->mac_address.u[3]).arg(it->mac_address.u[4]).arg(it->mac_address.u[5]);
		//unique access point key
		QString wlanx = ssid + mac;

		iRSSI = it->rssi;
		if (iRSSI == -100)
             iSig = 0;
         else if (iRSSI == -50)   
             iSig = 100;
         else
             iSig = (iRSSI+100)*2; 

		 if(iSig>100)iSig=100;

		 iLinkQuality=it->linkQuality;
		 iChannelFreq=(double)it->uChannelFreq/1000000.0;//GHz
		 iMaxDataTransRate=it->usRateSet;//Mbps

		 if(wlanx.isEmpty())
			 continue;

		 uLayerID = wlanlay.value(wlanx);
		 if(uLayerID==0)//add new AP
		 {
			//add layer
			qsl.clear();
			qsl << "-399" << "-399" << wlanx;
			pTree->slotAddObject(qsl,m_lOverlay,MST_ADD_LAYER,uLayerID);

			//set properties
			tmp.clear();
			tmp="IGNORE:CACHE=TRUE;DELETE=FALSE;EDIT=TRUE;O_DELETE=TRUE;O_EDIT=TRUE;FLAG=3;";
			pTree->slotSetObjectProperties(tmp,uLayerID,161);

			//pTree->slotLayerVisible(m_lLayer,wlanx,true);

			wlanlay.insert(wlanx,uLayerID);
		}
		 if(uLayerID!=0)//==m_lActiveLayer)
		 {
			pTree->slotLayerVisible(uLayerID,wlanx,true);
			//next group
			tmp = "MP-%1";
			tmp = tmp.arg(m_iCount);
			qsl.clear();
			qsl << "-399" << "-399" << tmp;
			pTree->slotAddObject(qsl,uLayerID,MST_ADD_GROUP,uGroupID);
			pTree->slotMoveObject(uGroupID,m_dGpsLon,m_dGpsLat);
			m_iCount++;

			QString qsTool = "SSID: %1;Signal: %2;RSSI: %3 dBm;LinkQuality: %4;ChannelFrequency: %5 GHz;TransferRate max: %6 Mbps";
			qsTool = qsTool.arg(ssid).arg(iSig).arg(iRSSI).arg(iLinkQuality).arg(iChannelFreq).arg(iMaxDataTransRate);

			int iPos=0;
			QList<QApMp*> pAllMP = m_qhMP.values(uLayerID);
			iPos = pAllMP.size();
			QApMp* pMP = new QApMp(uGroupID,m_dGpsLon,m_dGpsLat,iSig,iRSSI,iLinkQuality,iPos);
			m_qhMP.insertMulti(uLayerID,pMP);//add measure point

			//icon
			if(iSig>=70)
				qsMP = red;// + ";" + tmp;
			else if(iSig>=52)
				qsMP = orange;// + ";" + tmp;
			else if(iSig>=40)
				qsMP = oryel;
			else if(iSig>=28)
				qsMP = yellow;
			else
				qsMP = lblue;// + ";" + tmp;

			pTree->slotAddImageObject(qsMP,uGroupID,MST_ADD_ICON,0,0,0,-1,objectID,false);
			pTree->slotAddDescription(qsTool,uGroupID,111,1,0,objectID);//ToolTip
			objectID=0;
			tmp="Sig: %1 / RSSI: %2 dBm";
			tmp=tmp.arg(iSig).arg(iRSSI);
			pTree->slotAddDescription(tmp,uGroupID,0,0,0,objectID);//Text tree entry
			pTree->slotObjectVisible(objectID,false);
			
		}

	}
}

void MainWindow::scanStrongestAP()
{
	MarbleWidget* pWidget = m_controlView->marbleWidget();
	if(pWidget==NULL)return;

	TreeViewWidget* pTree = pWidget->tree();
	if(pTree==NULL)return;

	if(wlanMeasureAct->isChecked()&&m_bContinous)
		 QTimer::singleShot( m_iTimer*1000, this, SLOT( slotWlanMeasure() ) );
//	if(wlanMeasureAct->isChecked()&&!m_bContinous)
//		wlanMeasureAct->setChecked(false);

	if(!m_bGPS)return;//no position set

	wlanapi wi;// = new wlanapi();
	const ADAPTER_LIST &adapter_list = wi.get_adapter_list();
    const AP_LIST &ap_list = wi.get_ap_list(NULL);

	QStringList qsl;
	long overlayID =0;
	long uLayerID = 0;
	long uGroupID = 0;
	long objectID=0;
	double lat=0;
	double lon=0;
	char quality[64];
	short ccode=0;

	QString tmp;
	QString qsMP;
	QString path = QApplication::applicationDirPath();
	QString red("%1/red.png");
	QString orange("%1/orange.png");
	QString oryel("%1/oryel.png");
	QString yellow("%1/yellow.png");
	QString lblue("%1/lightblue.png");

	red = red.arg(path);
	orange = orange.arg(path);
	oryel = oryel.arg(path);
	yellow = yellow.arg(path);
	lblue = lblue.arg(path);

	//AP info	
	int iC = ap_list.size();
	int iSig = 0;
	int iRSSI= 0;
	int iLinkQuality=0;
	double iChannelFreq=0;
	int iMaxDataTransRate=0;
	//AP max values
	int iSigMax=0;
	int iRSSIMax=0;
	int iLinkQualityMax=0;
	double iChannelFreqMax=0;
	int iMaxDataTransRateMax=0;
	QString wlanMax;
	QString ssidMax;

	for (AP_LIST::const_iterator it = ap_list.begin(); it < ap_list.end(); ++it)
	{
		QString ssid = QString::fromUtf8((const char*)it->name);
		QString mac("\nMAC: %1-%2-%3-%4-%5-%6");
		mac = mac.arg(it->mac_address.u[0]).arg(it->mac_address.u[1]).arg(it->mac_address.u[2]).arg(it->mac_address.u[3]).arg(it->mac_address.u[4]).arg(it->mac_address.u[5]);
		//unique access point key
		QString wlanx = ssid + mac;

		iRSSI = it->rssi;
		if (iRSSI == -100)
			iSig = 0;
		else if (iRSSI == -50)   
			iSig = 100;
		else
			iSig = (iRSSI+100)*2; 

		if(iSig>100)iSig=100;
		iLinkQuality=it->linkQuality;
		iChannelFreq=(double)it->uChannelFreq/1000000.0;//GHz
		iMaxDataTransRate=it->usRateSet;//Mbps

		if(wlanx.isEmpty())
			continue;

		 if(iSig>iSigMax)
		 {
			 iSigMax=iSig;
			 iRSSIMax=iRSSI;
			 iLinkQualityMax=iLinkQuality;
			 iChannelFreqMax=iChannelFreq;
			 iMaxDataTransRateMax=iMaxDataTransRate;
			 ssidMax=ssid;
			 wlanMax = wlanx;
			 uLayerID = wlanlay.value(wlanx);
			 if(uLayerID==0)//add new AP
			 {
				//add layer
				qsl.clear();
				qsl << "-399" << "-399" << wlanx;
				pTree->slotAddObject(qsl,m_lOverlay,MST_ADD_LAYER,uLayerID);

				//set properties
				tmp.clear();
				tmp="IGNORE:CACHE=TRUE;DELETE=FALSE;EDIT=TRUE;O_DELETE=TRUE;O_EDIT=TRUE;FLAG=3;";
				pTree->slotSetObjectProperties(tmp,uLayerID,161);

				//pTree->slotLayerVisible(m_lLayer,wlanx,true);

				wlanlay.insert(wlanx,uLayerID);
			}
		 }
	}

	 if(uLayerID!=0)
	 {
		if(0)//m_lActiveLayer!=0)
		{
			QString wlan;
			pTree->slotLayerVisible(m_lActiveLayer,wlan,false);
		}

		//next group
		tmp = "MP-%1";
		tmp = tmp.arg(m_iCount);
		qsl.clear();
		qsl << "-399" << "-399" << tmp;
		pTree->slotAddObject(qsl,uLayerID,MST_ADD_GROUP,uGroupID);
		pTree->slotMoveObject(uGroupID,m_dGpsLon,m_dGpsLat);
		m_iCount++;

		QString qsTool = "SSID: %1;Signal: %2;RSSI: %3 dBm;LinkQuality: %4;ChannelFrequency: %5 GHz;TransferRate max: %6 Mbps";
		qsTool = qsTool.arg(ssidMax).arg(iSigMax).arg(iRSSIMax).arg(iLinkQualityMax).arg(iChannelFreqMax).arg(iMaxDataTransRateMax);

		int iPos=0;
		QList<QApMp*> pAllMP = m_qhMP.values(uLayerID);
		iPos = pAllMP.size();
		QApMp* pMP = new QApMp(uGroupID,m_dGpsLon,m_dGpsLat,iSigMax,iRSSIMax,iLinkQualityMax,iPos);
		m_qhMP.insertMulti(uLayerID,pMP);//add measure point

		//icon
		if(iSigMax>=70)
			qsMP = red;// + ";" + tmp;
		else if(iSigMax>=52)
			qsMP = orange;// + ";" + tmp;
		else if(iSigMax>=40)
			qsMP = oryel;
		else if(iSigMax>=28)
			qsMP = yellow;
		else
			qsMP = lblue;// + ";" + tmp;

		pTree->slotAddImageObject(qsMP,uGroupID,MST_ADD_ICON,0,0,0,-1,objectID,false);
		pTree->slotAddDescription(qsTool,uGroupID,111,1,0,objectID);
		objectID=0;
		tmp="Sig: %1 / RSSI: %2 dBm";
		tmp=tmp.arg(iSigMax).arg(iRSSIMax);
		pTree->slotAddDescription(tmp,uGroupID,0,0,0,objectID);
		pTree->slotObjectVisible(objectID,false);
		m_lActiveLayer=uLayerID;
		pTree->slotLayerVisible(m_lActiveLayer,wlanMax,true);
	}
}

void MainWindow::setMessPoint( GeoDataCoordinates pos)
{
	if(!wlanMeasureAct->isChecked())
		return;

	if(!m_bMP)//use GPS values
	{
		m_dGpsLon = pos.longitude(GeoDataCoordinates::Degree);
		m_dGpsLat = pos.latitude(GeoDataCoordinates::Degree);
	}

	if(m_dGpsLon!=0 || m_dGpsLat!=0)
		m_bGPS=true;
		
	if(m_bGPS)
	{
		//wlanMeasureAct->setEnabled(m_bGPS);
		if(!m_bContinous)
			slotWlanMeasure();
	}
}

void MainWindow::slotGps()
{
	m_controlView->showRoutingWidget(this);
}

void MainWindow::receiveGpsCoordinates( GeoDataCoordinates pos, qreal speed)
{
	m_dGpsLon = pos.longitude(GeoDataCoordinates::Degree);
	m_dGpsLat = pos.latitude(GeoDataCoordinates::Degree);

	if(m_dGpsLon!=0 || m_dGpsLat!=0)
		m_bGPS=true;
		
	if(m_bMP)//&&m_lActiveLayer>0)
		wlanMeasureAct->setEnabled(m_bGPS);
}

void MainWindow::slotHomeChanged( GeoDataCoordinates pos)
{
/*	m_dGpsLon = pos.longitude(GeoDataCoordinates::Degree);
	m_dGpsLat = pos.latitude(GeoDataCoordinates::Degree);

	if(m_dGpsLon!=0 || m_dGpsLat!=0)
		m_bGPS=true;
		
	if(m_lActiveLayer>0)
	{
		wlanMeasureAct->setEnabled(m_bGPS);
		if(!m_bContinous&&m_bGPS)
			slotWlanMeasure();
	}*/
}

void MainWindow::slotSelectedItem(long overlayID,long layerID,int id)
{
	long iid=id;
}

void MainWindow::slotSettings()
{
	QDialog dlg;
	m_uiWlanSettings = new Ui::qSettings;
	m_uiWlanSettings->setupUi( &dlg );

	m_uiWlanSettings->qContinous->setChecked(m_bContinous);
	m_uiWlanSettings->qSingle->setChecked(!m_bContinous);
	m_uiWlanSettings->qTimer->setValue(m_iTimer);

	switch(m_iAP)
	{
		case 0:
			m_uiWlanSettings->qSelectedAP->setChecked(true);
			m_uiWlanSettings->qStrongestAP->setChecked(false);
			m_uiWlanSettings->qAllAP->setChecked(false);
			break;
		case 1:
			m_uiWlanSettings->qSelectedAP->setChecked(false);
			m_uiWlanSettings->qStrongestAP->setChecked(true);
			m_uiWlanSettings->qAllAP->setChecked(false);
			break;
		case 2:
			m_uiWlanSettings->qSelectedAP->setChecked(false);
			m_uiWlanSettings->qStrongestAP->setChecked(false);
			m_uiWlanSettings->qAllAP->setChecked(true);
			break;
		default:
			m_uiWlanSettings->qSelectedAP->setChecked(true);
			m_uiWlanSettings->qStrongestAP->setChecked(false);
			m_uiWlanSettings->qAllAP->setChecked(false);
			break;
	}

	m_uiWlanSettings->qGPS->setChecked(m_bMP);
	m_uiWlanSettings->qManual->setChecked(!m_bMP);

	connect(m_uiWlanSettings->qTimer,SIGNAL(valueChanged(int)),this,SLOT(slotTimer(int)));
	connect(m_uiWlanSettings->qContinous,SIGNAL(toggled(bool)),this,SLOT(slotContinous(bool)));
	connect(m_uiWlanSettings->qSingle,SIGNAL(toggled(bool)),this,SLOT(slotSingle(bool)));

	connect(m_uiWlanSettings->qSelectedAP,SIGNAL(toggled(bool)),this,SLOT(slotSelectedAP(bool)));
	connect(m_uiWlanSettings->qStrongestAP,SIGNAL(toggled(bool)),this,SLOT(slotStrongestAP(bool)));
	connect(m_uiWlanSettings->qAllAP,SIGNAL(toggled(bool)),this,SLOT(slotAllAP(bool)));

	connect(m_uiWlanSettings->qGPS,SIGNAL(toggled(bool)),this,SLOT(slotMPgps(bool)));
	connect(m_uiWlanSettings->qManual,SIGNAL(toggled(bool)),this,SLOT(slotMPmanual(bool)));

	if(m_uiWlanSettings!=NULL)
	{
		dlg.exec();
		showWlanSettings();
	}

	delete m_uiWlanSettings;
}

void MainWindow::slotTimer(int value)
{
	m_iTimer=value;
	showWlanSettings();
}	
void MainWindow::slotContinous(bool state)
{
	m_bContinous=state;
	if(!m_bMP && m_iAP>0 && wlanlay.size()>0)
		wlanMeasureAct->setEnabled(true);
	showWlanSettings();
}	
void MainWindow::slotSingle(bool state)
{
	m_bContinous=!state;
	if(!m_bMP && m_iAP>0 && wlanlay.size()>0)
		wlanMeasureAct->setEnabled(true);
	showWlanSettings();
/*	if(state)
	{
		if(!wlanMeasureAct->isChecked()&&wlanlay.size()>0)
			wlanMeasureAct->setEnabled(true);
	}*/
}
void MainWindow::slotSelectedAP(bool state)
{
	if(state)m_iAP=0;
	if(!m_bMP && m_lActiveLayer!=0 && wlanlay.size()>0)
		wlanMeasureAct->setEnabled(true);
	showWlanSettings();
}
void MainWindow::slotStrongestAP(bool state)
{
	if(state)m_iAP=1;
	if(!m_bMP && wlanlay.size()>0)
		wlanMeasureAct->setEnabled(true);
	showWlanSettings();
}
void MainWindow::slotAllAP(bool state)
{
	if(state)m_iAP=2;
	if(!m_bMP && wlanlay.size()>0)
		wlanMeasureAct->setEnabled(true);
	showWlanSettings();
}

void MainWindow::slotMPgps(bool state)
{
	m_bMP=state;
	wlanMeasureAct->setEnabled(false);
	showWlanSettings();
}
	
void MainWindow::slotMPmanual(bool state)
{
	m_bMP=!state;
	if(state)wlanMeasureAct->setEnabled(true);
	showWlanSettings();
}

void MainWindow::slotSendEvent(long type,long size,void* pData,long intID)
{
	char* pValue = (char*)pData;
	QByteArray ba;
	ba.setRawData(pValue,size);
	QDataStream s(ba);
	s.setByteOrder(QDataStream::LittleEndian);  //for MapView

	//set message type and size
	uint objID=0;

	switch(type)
	{
		case MST_EVENT_WW:
			{
				s >> objID;
				double lon1=0;
				double lat1=0;
				double lon2=0;
				double lat2=0;
				s >> lon1;
				s >> lat1;
				s >> lon2;
				s >> lat2;
			}
			break;
		case MST_EVENT_VISIBLE_WW:
			{
				s >> objID;
				double lon1=0;
				double lat1=0;
				double lon2=0;
				double lat2=0;
				s >> lon1;
				s >> lat1;
				s >> lon2;
				s >> lat2;
			}
			break;
		case MST_EVENT_GROUP_SELECT:
			{
				s >> objID;
			}
			break;
		case MST_EVENT_GROUP_DESELECT:
			{
				s >> objID;
			}
			break;
		case MST_EVENT_GROUP_DELETE:
			{
				s >> objID;
			}
			break;
		case MST_EVENT_GROUP_MOVE:
			{
				s >> objID;
				double lon=0;
				double lat=0;
				s >> lon;
				s >> lat;

			}
			break;

		case MST_EVENT_GROUP_MOVE_EX:
			{
				s >> objID;
				double lon1=0;
				double lat1=0;
				double lon2=0;
				double lat2=0;
				s >> lon1;
				s >> lat1;
				s >> lon2;
				s >> lat2;
	
			}
			break;

		case MST_EVENT_POLY_CREATE:
			{
				struct t_Data {
						long shapeptr;
						long count;
						void* pData;
				} ;

				t_Data* pSendData=NULL;
				pSendData=(t_Data*)ba.data();

				objID = pSendData->shapeptr;

			}
			break;

		case MST_EVENT_MENU_ID:
			{
				s >> objID;
			}
			break;

		case MST_EVENT_LAYER_OBJECTS:
			{
			
			}
			break;

		case MST_EVENT_MOUSE_ACT:
			{
				s >> objID;

				int type=0;
				double lon=0;
				double lat=0;
				int xpos=0;
				int ypos=0;

				s >> type;
				s >> lon;
				s >> lat;
				s >> xpos;
				s >> ypos;

				if( (type==1||type==101)&&wlanlay.size()>0)//left mouse button - AP list not empty
				{
					GeoDataCoordinates pos;
					pos.set(lon,lat,0,GeoDataCoordinates::Degree);
					if(m_iAP==0&&m_lActiveLayer!=0)//do we have an AP selection?
						setMessPoint(pos);
					else if(m_iAP>0)
						setMessPoint(pos);
				}

			}
			break;
		
		case MST_EVENT_MOUSE_OVER:
			{
				s >> objID;

				double section=0;
				double xpos=0;
				double ypos=0;
				s >> section;
				s >> xpos;
				s >> ypos;
			}
			break;

		case MST_EVENT_OBJECT_VISIBLE:
			{
				struct t_Data{
						long objectID;
						long visible;
						long size;
						void* pData;
				};	

				t_Data* pSendData=NULL;
				pSendData=(t_Data*)ba.data();
				QString wlanx((char*)pSendData->pData);
				setLayerVisible(pSendData->objectID,pSendData->visible,wlanx);
			}
			break;

		default:break;
	}
}

void MainWindow::setLayerVisible(long id,long visible,QString& wlanx)
{
	MarbleWidget* pWidget = m_controlView->marbleWidget();
	if(pWidget==NULL)return;

	TreeViewWidget* pTree = pWidget->tree();
	if(pTree==NULL)return;

	if(!wlanlay.contains(wlanx))return;

	if(m_lActiveLayer>0 && m_lActiveLayer!=id&&m_iAP==0)//disable selected layer for AP mode: selected
	{
		pTree->slotLayerVisible(m_lActiveLayer,wlanx,false);
		m_iCount=0;
	}
	if(visible)
	{
		m_lActiveLayer=id;
		if(m_bMP)//enable if GPS signal is valid
			wlanMeasureAct->setEnabled(m_bGPS);
		else if(m_iAP==0)//enable for AP mode: selected and MP mode: manual
			wlanMeasureAct->setEnabled(true);
	}
	else m_lActiveLayer=0;
}

void MainWindow::slotCreateHeatmap()
{
	if(m_lActiveLayer==0)return;

	if(m_qhMP.size()<1)return;

	MarbleWidget* pWidget = m_controlView->marbleWidget();
	if(pWidget==NULL)return;

	TreeViewWidget* pTree = pWidget->tree();
	if(pTree==NULL)return;

	//iterate all wlan layers
	long id=0;
	QString wlanx;
    QHash<QString,long>::const_iterator iter = wlanlay.constBegin();
    for ( ; iter != wlanlay.constEnd(); ++iter ) 
	{
		id = iter.value();
		wlanx = iter.key();
		if(id==m_lActiveLayer)
			break;
	}

	if(id==0)return;

	QList<QApMp*> pAllMP = m_qhMP.values(id);
	if(pAllMP.size()<1)return;

	QStringList qsl;
	QString tmp = "Heatmap";
	if(!shadinglay.contains(tmp))
	{
		qsl << "-399" << "-399" << tmp;
		pTree->slotAddObject(qsl,0,MST_ADD_OVERLAY,m_lShadingOverlay);
		shadinglay.insert(tmp,m_lShadingOverlay);
	}

	//add layer
	qsl.clear();
	if(!shadinglay.contains(wlanx))
	{
		qsl << "-399" << "-399" << wlanx;
		pTree->slotAddObject(qsl,m_lShadingOverlay,MST_ADD_SHADING_LAYER,m_lShadingLayer);

		//set properties
		tmp.clear();
		tmp="IGNORE:CACHE=TRUE;DELETE=FALSE;EDIT=TRUE;O_DELETE=TRUE;O_EDIT=TRUE;FLAG=3;";
		pTree->slotSetObjectProperties(tmp,m_lShadingLayer,161);

		shadinglay.insert(wlanx,m_lShadingLayer);
	}
	else m_lShadingLayer = shadinglay.value(wlanx);
	
	GeoDataLineString gdl;
	GeoDataCoordinates pos1;
	GeoDataCoordinates pos2;

	double dm1 = 0.0000125;//1m
	double length = 0;
	double left=0;
	double top=0;
	double right=0;
	double bott=0;

	QColor red(255,0,0,80);
	QColor orange(255,128,0,80);
	QColor yelora(248,177,71,80);
	QColor yellow(255,255,0,80);
	QColor lblue(0,255,255,80);

	uint uRed = red.alpha();
	uRed <<=8;
	uRed += red.red();
	uRed <<=8;
	uRed += red.green();
	uRed <<=8;
	uRed += red.blue();

	uint uOrange = orange.alpha();
	uOrange <<=8;
	uOrange += orange.red();
	uOrange <<=8;
	uOrange += orange.green();
	uOrange <<=8;
	uOrange += orange.blue();

	uint uYelora = yelora.alpha();
	uYelora <<=8;
	uYelora += yelora.red();
	uYelora <<=8;
	uYelora += yelora.green();
	uYelora <<=8;
	uYelora += yelora.blue();

	uint uYellow = yellow.alpha();
	uYellow <<=8;
	uYellow += yellow.red();
	uYellow <<=8;
	uYellow += yellow.green();
	uYellow <<=8;
	uYellow += yellow.blue();

	uint uLblue = lblue.alpha();
	uLblue <<=8;
	uLblue += lblue.red();
	uLblue <<=8;
	uLblue += lblue.green();
	uLblue <<=8;
	uLblue += lblue.blue();

	qint32 col=0;
	int grid=10;
	long objectID=0;
	long gridID=0;

	QApMp* pApMp=NULL;
	QApMp* pApMpNext=NULL;
	int iCount = pAllMP.size();

	for(int i=0;i<iCount;i++)
	{
		pApMp = pAllMP.at(i);
		//if(i+1<iCount)
		//	pApMpNext = pAllMP.at(i+1);

		if(pApMp!=NULL)
		{
			int iPos=pApMp->getPosition();
			if(iPos<iCount-1)
			{
				for(int ix=0;ix<iCount;ix++)
				{
					pApMpNext = pAllMP.at(ix);
					if(pApMpNext!=NULL && pApMpNext->getPosition()==iPos+1)
					{
						pos1.set(pApMp->getLongitude(),pApMp->getLatitude(),0,GeoDataCoordinates::Degree);
						pos2.set(pApMpNext->getLongitude(),pApMpNext->getLatitude(),0,GeoDataCoordinates::Degree);
						gdl << pos1 << pos2;
						length = ((gdl.length(pWidget->model()->planet()->radius()))/2)*dm1;
						ix=iCount;
					}
				}
			}
			else
			{
				for(int ix=0;ix<iCount;ix++)
				{
					pApMpNext = pAllMP.at(ix);
					if(pApMpNext!=NULL && pApMpNext->getPosition()==iPos-1)
					{
						pos1.set(pApMp->getLongitude(),pApMp->getLatitude(),0,GeoDataCoordinates::Degree);
						pos2.set(pApMpNext->getLongitude(),pApMpNext->getLatitude(),0,GeoDataCoordinates::Degree);
						gdl << pos1 << pos2;
						length = ((gdl.length(pWidget->model()->planet()->radius()))/2)*dm1;
						ix=iCount;
					}
				}
			}
			//length = dm1*1;
			gdl.clear();

			left = pApMp->getLongitude()-length;
			right = pApMp->getLongitude()+length;
			top = pApMp->getLatitude()+length;
			bott = pApMp->getLatitude()-length;

			pTree->slotAddShadingParameter(gridID,grid,grid,left,top,right,bott,m_lShadingLayer);

			if(pApMp->getSignal()>=70)
				col = uRed;
			else if(pApMp->getSignal()>=52)
				col = uOrange;
			else if(pApMp->getSignal()>=40)
				col = uYelora;
			else if(pApMp->getSignal()>=28)
				col = uYellow;
			else
				col = uLblue;

			QByteArray qba;
			QDataStream stream( &qba, QIODevice::WriteOnly);
			stream.setByteOrder(QDataStream::LittleEndian);
			for(int y=0;y<grid;y++)
			{
				for(int x=0;x<grid;x++)
				{
					stream << col;
				}
			}

			char* pData = qba.data();
			pTree->slotAddColoredAreaData(m_lShadingLayer,gridID,grid*grid,(void*)pData,objectID);

			//pApMp->getLinkQuality();
			//pApMp->getRSSI();
			gridID=0;
			objectID=0;
			pApMpNext=NULL;
			pApMp=NULL;
		}
	}
}

void MainWindow::updateStatusBar()
{
	if(m_infoLabel)
	{
		if(!m_info.isEmpty())
			m_infoLabel->setText(m_info);
	}

    if ( m_positionLabel )
        m_positionLabel->setText( QString( "%1 %2" ).
        arg( tr( POSITION_STRING ) ).arg( m_position ) ); 

    if ( m_distanceLabel )
        m_distanceLabel->setText( QString( "%1 %2" )
        .arg( tr( DISTANCE_STRING ) ).arg( m_distance ) ); 

	if(m_elevationLabel)
		m_elevationLabel->setText( QString( "%1 %2" )
        .arg( tr( ELEVATION_STRING ) ).arg( m_elevation ) ); 

	m_elevation.clear();
	//m_elevation = "n.a.";
}

void MainWindow::setupStatusBar()
{
    statusBar()->setSizeGripEnabled( true );

	m_infoLabel = new QLabel( );
    m_infoLabel->setIndent( 5 );
    QString templateString = QString( "Info pane" );
    int maxWidth = fontMetrics().boundingRect(templateString).width()
                            + 2 * m_infoLabel->margin() + 2 * m_infoLabel->indent()+500;
    m_infoLabel->setFixedWidth( maxWidth+20 );
	m_infoLabel->setStyleSheet("QLabel{color: red;}");
    statusBar()->addPermanentWidget ( m_infoLabel );

    m_positionLabel = new QLabel( );
    m_positionLabel->setIndent( 5 );
    QString templatePositionString = 
        QString( "%1 000\xb0 00\' 00\"_, 000\xb0 00\' 00\"_" ).arg(POSITION_STRING);
    int maxPositionWidth = fontMetrics().boundingRect(templatePositionString).width()
                            + 2 * m_positionLabel->margin() + 2 * m_positionLabel->indent()+10;
    m_positionLabel->setFixedWidth( maxPositionWidth+20 );
    statusBar()->addPermanentWidget ( m_positionLabel );

    m_distanceLabel = new QLabel( );
    m_distanceLabel->setIndent( 5 );
    QString templateDistanceString = 
        QString( "%1 00.000,0 mu" ).arg(DISTANCE_STRING);
    int maxDistanceWidth = fontMetrics().boundingRect(templateDistanceString).width()
                            + 2 * m_distanceLabel->margin() + 2 * m_distanceLabel->indent();
    m_distanceLabel->setFixedWidth( maxDistanceWidth );
    statusBar()->addPermanentWidget ( m_distanceLabel );

	m_elevationLabel = new QLabel( );
    m_elevationLabel->setIndent( 5 );
	QString templateElevationString = 
        QString( "%1 00.00 m" ).arg(ELEVATION_STRING);
    int maxElevationWidth = fontMetrics().boundingRect(templateElevationString).width()
                            + 2 * m_elevationLabel->margin() + 2 * m_elevationLabel->indent();
    m_elevationLabel->setFixedWidth( maxElevationWidth );
    statusBar()->addPermanentWidget ( m_elevationLabel );

	MarbleWidget* pWidget = m_controlView->marbleWidget();
	if(pWidget==NULL)return;

    connect( pWidget, SIGNAL( mouseMoveGeoPosition( QString ) ),
              this, SLOT( showPosition( QString ) ) );
    connect( pWidget, SIGNAL( distanceChanged( QString ) ),
              this, SLOT( showDistance( QString ) ) );

	TreeViewWidget* pTree = pWidget->tree();
	if(pTree==NULL)return;

	connect( pTree, SIGNAL( signalElevationValue(qreal) ),
              this, SLOT( showElevationValue(qreal) ) );
//	connect( pTree, SIGNAL( signalCoordinateFormat(qreal,qreal,long) ),
//              this, SLOT( slotCoordinateFormat(qreal,qreal,long) ) );

    updateStatusBar();
}

void MainWindow::showPosition( const QString& position )
{
    m_position = position;
//	m_lObjectID=0;

	if(0)//position != "not available" )
	{
		if(1)//m_cfDialog!=NULL)
		{
			QString qsLon = position.section(',',0,0);
			QString qsLat = position.section(',',1,1);

			QChar chLat = qsLat.at(qsLat.length()-1);
			QChar chLon = qsLon.at(qsLon.length()-1);
			qsLat = qsLat.left(qsLat.length()-1);
			qsLon = qsLon.left(qsLon.length()-1);

		//	qsLat = qsLat.right(qsLat.length()-2);
		//	qsLon = qsLon.right(qsLon.length()-2);
			qsLat = qsLat.trimmed();
			qsLon = qsLon.trimmed();

			qsLat.insert(0,chLat);
			qsLon.insert(0,chLon);

			m_position.clear();
			m_position = qsLon;
			m_position += "  ";
			m_position += qsLat;

		}

	}

    updateStatusBar();
}

void MainWindow::showDistance( const QString& distance )
{
    m_distance = distance;
    updateStatusBar();
}

void MainWindow::showElevationValue(qreal elev )
{
	QString qs("%1 m");
	if(elev != -32768)
		m_elevation = qs.arg(elev);
	else m_elevation.clear();
    updateStatusBar();
}

void MainWindow::showWlanSettings()
{
	m_info.clear();
	QString scanmode;
	if(m_bContinous)
		scanmode="scanmode: continous / ";
	else scanmode="scanmode: single shot / ";
	QString apmode;
	switch(m_iAP)
	{
		case 0: apmode="access point mode: selected / ";
			    break;
		case 1: apmode="access point mode: signal strength / ";
			    break;
		case 2: apmode="access point mode: all / ";
			    break;
	}
	QString mpset;
	if(m_bMP)
		mpset="measure point set by GPS";
	else mpset="measure point set by left mouse click";

	m_info = scanmode + apmode + mpset;
}

void MainWindow::slotCoordinateFormat(qreal,qreal,long)
{

}

void MainWindow::setProjection(int iProj)
{
	if(m_iProjection == iProj)
		return;

	MarbleWidget* pWidget = m_controlView->marbleWidget();
	if(pWidget==NULL)return;

	Marble::Projection prj = Marble::Projection::Equirectangular;
	m_iProjection = iProj;

	switch(iProj)
	{
		case 0: prj = Marble::Projection::Spherical;break;
		case 2: prj = Marble::Projection::Mercator;break;
		case 1: prj = Marble::Projection::Equirectangular;break;
		default:prj = Marble::Projection::Spherical;break;
	}

	pWidget->setProjection(prj);
}

void MainWindow::loadFile(const QString &fileName)
{

    QFile file(fileName);
    if (!file.open(QFile::ReadOnly | QFile::Text)) {
        QMessageBox::warning(this, tr("WlanMap"),
                             tr("Cannot read file %1:\n%2.")
                             .arg(fileName)
                             .arg(file.errorString()));
        return;
    }

    statusBar()->showMessage(tr("File loaded"), 2000);
}

bool MainWindow::saveFile(const QString &fileName)
{
    QFile file(fileName);
    if (!file.open(QFile::WriteOnly | QFile::Text)) {
        QMessageBox::warning(this, tr("SDI"),
                             tr("Cannot write file %1:\n%2.")
                             .arg(fileName)
                             .arg(file.errorString()));
        return false;
    }

    statusBar()->showMessage(tr("File saved"), 2000);
    return true;
}

void MainWindow::deleteMapDirectory(QString& mapdir)
{
	QDir map(mapdir);
	if(!map.exists())
		return;

	int iIdx = mapdir.lastIndexOf('/')+1;
	QString tmp = mapdir.right(mapdir.length()-iIdx);
	int iVal = tmp.toInt();
	if(iVal==0)
	{
		tmp = mapdir.left(iIdx-1);
		tmp = tmp.right(1);
		iVal = tmp.toInt();
	}

	//First delete any files in the current directory
	QFileInfoList files = map.entryInfoList(QDir::NoDotAndDotDot | QDir::Files);
	if(iVal>0)
	{
 
		for(int file = 0; file < files.count(); file++)
		{
			map.remove(files.at(file).fileName());
		}
	}
 
	//Now recursively delete any child directories
	QFileInfoList dirs = map.entryInfoList(QDir::NoDotAndDotDot | QDir::Dirs);
	for(int dir = 0; dir < dirs.count(); dir++)
	{
		this->deleteMapDirectory(dirs.at(dir).absoluteFilePath ());
	}
 
	//Finally, remove empty parent directory
	if(iVal>0)
		map.rmdir(map.path());
 }

#include "moc_mainwindow.cpp"