//
// This file is part of the Marble Virtual Globe.
//
// This program is free software licensed under the GNU LGPL. You can
// find a copy of this license in LICENSE.txt in the top directory of
// the source code.
//
// Copyright 2007   Andrew Manson   <g.real.ate@gmail.com>
// Copyright 2009   Eckhart Wörner  <ewoerner@kde.org>
// Copyright 2010   Thibaut Gridel  <tgridel@free.fr>
// Copyright 2010   Daniel Marth    <danielmarth@gmx.at>
//

#include "PositionMarker.h"

#include "MarbleDebug.h"
#include <QtCore/QRect>
#include <QtCore/qmath.h>
#include <QtGui/QFileDialog>
#include <QtGui/QMessageBox>
#include <QtGui/QPushButton>
#include <QtGui/QColorDialog>
#include <QtGui/QTransform>

#include <cmath>

#include "uic_PositionMarkerConfigWidget.h"
#include "MarbleWidget.h"
#include "MarbleModel.h"
#include "MarbleMap.h"
#include "MarbleDirs.h"
#include "GeoPainter.h"
#include "PositionTracking.h"
#include "ViewportParams.h"
#include "GeoDataPolygon.h"
#include "GeoDataLineString.h"
#include "Planet.h"

namespace Marble
{

const int PositionMarker::sm_defaultSizeStep = 2;
const float PositionMarker::sm_resizeSteps[] = { 0.25, 0.5, 1.0, 2.0, 4.0 };
const int PositionMarker::sm_numResizeSteps = sizeof( sm_resizeSteps ) / sizeof( sm_resizeSteps[0] );

PositionMarker::PositionMarker ()
    : RenderPlugin( 0 ),
      ui_configWidget( 0 ),
      m_configDialog( 0 )
{
}

PositionMarker::PositionMarker( const MarbleModel *marbleModel )
    : RenderPlugin( marbleModel ),
      m_isInitialized( false ),
      m_useCustomCursor( false ),
      m_defaultCursorPath( MarbleDirs::path( "svg/track_turtle.svg" ) ),
      m_lastBoundingBox(),
      ui_configWidget( 0 ),
      m_configDialog( 0 ),
	  m_cursorPath( m_defaultCursorPath ),
      m_cursorSize( 1.0 ),
	  m_accuracyColor( oxygenBrickRed4 ),
      m_trailColor( 0, 0, 255 ),
      m_heading( 0.0 ),
      m_showTrail ( false ),
	  m_marbleWidget(NULL)
{
	 m_cursorSize = sm_resizeSteps[2];//enz
	 m_lOverlay=0;
	 m_lLayer=0;
	 m_lGroup=0;
	 m_lPoly=0;

	 m_prevHeading   = 0;
	 m_prevLon       = 399.0;
	 m_prevLat       = 399.0;
	 m_prevSpeed     = 0;
	 m_prevDistance  = 0;

	 m_gpsTrack = (GeoDataLineString*) new GeoDataLineString();

	 const bool smallScreen = MarbleGlobal::getInstance()->profiles() & MarbleGlobal::SmallScreen;
    m_accuracyColor.setAlpha( smallScreen ? 80 : 40 );
}

PositionMarker::~PositionMarker ()
{
    delete ui_configWidget;
    delete m_configDialog;

	if( m_gpsTrack!=NULL)
		delete  m_gpsTrack;
}

QStringList PositionMarker::renderPosition() const
{
    return QStringList( "HOVERS_ABOVE_SURFACE" );
}

QString PositionMarker::renderPolicy() const
{
    return "ALWAYS";
}

QStringList PositionMarker::backendTypes() const
{
    return QStringList( "positionmarker" );
}

QString PositionMarker::name() const
{
    return tr( "Position Marker" );
}

QString PositionMarker::guiString() const
{
    return tr( "&Position Marker" );
}

QString PositionMarker::nameId() const
{
    return QString( "positionMarker" );
}

QString PositionMarker::version() const
{
    return "1.0";
}

QString PositionMarker::description() const
{
    return tr( "draws a marker at the current position" );
}

QString PositionMarker::copyrightYears() const
{
    return "2009, 2010";
}

QList<PluginAuthor> PositionMarker::pluginAuthors() const
{
    return QList<PluginAuthor>()
            << PluginAuthor( "Andrew Manson", "g.real.ate@gmail.com" )
            << PluginAuthor( "Eckhart Woerner", "ewoerner@kde.org" )
            << PluginAuthor( "Thibaut Gridel", "tgridel@free.fr" )
            << PluginAuthor( "Daniel Marth", "danielmarth@gmx.at" );
}

QIcon PositionMarker::icon() const
{
    return QIcon();
}

QDialog *PositionMarker::configDialog()
{
    if ( !m_configDialog ) {
        // Initializing configuration dialog
        m_configDialog = new QDialog();
        ui_configWidget = new Ui::PositionMarkerConfigWidget;
        ui_configWidget->setupUi( m_configDialog );
        ui_configWidget->m_resizeSlider->setMaximum( sm_numResizeSteps - 1 );
        readSettings();
        connect( ui_configWidget->m_buttonBox, SIGNAL( accepted() ),
                 SLOT( writeSettings() ) );
        connect( ui_configWidget->m_buttonBox, SIGNAL( rejected() ),
                 SLOT( readSettings() ) );
        connect( ui_configWidget->m_buttonBox->button( QDialogButtonBox::RestoreDefaults ), SIGNAL( clicked () ),
                 SLOT( restoreDefaultSettings() ) );
        QPushButton *applyButton = ui_configWidget->m_buttonBox->button( QDialogButtonBox::Apply );
        connect( applyButton, SIGNAL( clicked() ),
                 SLOT( writeSettings() ) );
        connect( ui_configWidget->m_fileChooserButton, SIGNAL( clicked() ),
                 SLOT( chooseCustomCursor() ) );
        connect( ui_configWidget->m_resizeSlider, SIGNAL( valueChanged( int ) ),
                 SLOT( resizeCursor( int ) ) );
        connect( ui_configWidget->m_acColorChooserButton, SIGNAL( clicked() ),
                 SLOT( chooseColor() ) );
        connect( ui_configWidget->m_trailColorChooserButton, SIGNAL( clicked() ),
                 SLOT( chooseColor() ) );
    }
    return m_configDialog;
}

void PositionMarker::initialize()
{
    if ( marbleModel() ) {
        connect( marbleModel()->positionTracking(), SIGNAL( gpsLocation( GeoDataCoordinates,qreal ) ),
                this, SLOT( setPosition( GeoDataCoordinates ) ) );

		connect( marbleModel()->positionTracking(), SIGNAL( positionProviderDisabled(bool) ),
                this, SLOT( positionProviderDisabled(bool) ) );
		
        m_isInitialized = true;
    }
    loadDefaultCursor();
}

bool PositionMarker::isInitialized() const
{
    return m_isInitialized;
}


void PositionMarker::update( const ViewportParams *viewport )
{
    if( ! ( m_currentPosition == m_previousPosition ) )
    {
        QPointF position;
        QPointF previousPosition;

        viewport->screenCoordinates( m_currentPosition, position );
        viewport->screenCoordinates( m_previousPosition, previousPosition );

        // calculate the arrow shape, oriented by the heading
        // and with constant size
        QPointF unitVector = position - previousPosition;
        // check that some screen progress was made
        if( unitVector.x() || unitVector.y() ) {
            // magnitude should be >0
            qreal magnitude = sqrt( ( unitVector.x() * unitVector.x() )
                                    + ( unitVector.y() * unitVector.y() ) );
            m_heading = atan( unitVector.y() / unitVector.x() ) + M_PI / 2.0;
            unitVector = unitVector / magnitude;
            QPointF unitVector2 = QPointF ( -unitVector.y(), unitVector.x() );
            QPointF relativeLeft = - ( unitVector * 9   ) + ( unitVector2 * 9 );
            QPointF relativeRight = - ( unitVector * 9 ) - ( unitVector2 * 9 );
            QPointF relativeTip =  unitVector * 19.0 ;

            m_arrow.clear();
            m_arrow << position
                    << position + ( relativeLeft * m_cursorSize )
                    << position + ( relativeTip * m_cursorSize )
                    << position + ( relativeRight * m_cursorSize );

            m_dirtyRegion = QRegion();
            m_dirtyRegion += ( m_arrow.boundingRect().toRect() );
            m_dirtyRegion += ( m_previousArrow.boundingRect().toRect() );

            // Check if position has changed.
            m_trail.push_front( m_currentPosition );
            for( int i = sm_numTrailPoints + 1; i< m_trail.size(); ++i ) {
                    m_trail.pop_back();
            }
        }
    }
}

bool PositionMarker::render( GeoPainter *painter,
                           ViewportParams *viewport,
                           const QString& renderPos,
                           GeoSceneLayer * layer )
{
    Q_UNUSED( layer )
    bool const gpsActive = marbleModel()->positionTracking()->positionProviderPlugin() != 0;
	static bool turn=false;
    if ( gpsActive && renderPosition().contains(renderPos) )
    {
		m_lastBoundingBox = viewport->viewLatLonAltBox();
        update( viewport );
        painter->save();
        painter->autoMapQuality();

        GeoDataAccuracy accuracy = marbleModel()->positionTracking()->accuracy();
        if ( accuracy.horizontal > 0 && accuracy.horizontal < 1000 ) {
            // Paint a circle indicating the position accuracy
            painter->setPen( Qt::transparent );
            int width = qRound( accuracy.horizontal * viewport->radius() / EARTH_RADIUS );
            if ( MarbleGlobal::getInstance()->profiles() & MarbleGlobal::SmallScreen ) {
                int arrowSize = qMax<int>( m_arrow.boundingRect().width(), m_arrow.boundingRect().height() );
                width = qMax<int>( width, arrowSize + 10 );
            }

            painter->setBrush( m_accuracyColor );
            painter->drawEllipse( m_currentPosition, width, width );
        }

        // Draw trail if requested.
        if( m_showTrail ) {
            painter->save();
            // Use selected color to draw trail.
            painter->setBrush( m_trailColor );
            painter->setPen( m_trailColor );
            QRectF trailRect;
            QPointF trailPoint;
            float opacity = 1.0;
            // we don't draw m_trail[0] which is current position
            for( int i = 1; i < m_trail.size(); ++i ) {
                // Get screen coordinates from coordinates on the map.
                viewport->screenCoordinates( m_trail[i], trailPoint );
                int size = ( sm_numTrailPoints - i ) * 3;
                trailRect.setX( trailPoint.x() - size / 2.0 );
                trailRect.setY( trailPoint.y() - size / 2.0 );
                trailRect.setWidth( size );
                trailRect.setHeight( size );
                painter->setOpacity( opacity );
                painter->drawEllipse( trailRect );
                opacity -= 0.15;
            }
            painter->restore();
        }

        if( m_useCustomCursor)
        {
            QRect rect = m_arrow.boundingRect().toRect();
            if( rect.isValid() )
            {
                QTransform transform;
                transform.translate( -m_customCursor.width() / 2, -m_customCursor.height() / 2 );
                transform.rotateRadians( m_heading );
                transform.translate( m_customCursor.width() / 2, m_customCursor.height() / 2 );
                if( painter->mapQuality() == HighQuality || painter->mapQuality() == PrintQuality )
                    painter->drawPixmap( rect.topLeft(), m_customCursor.transformed( transform, Qt::SmoothTransformation ) );
                else
                    painter->drawPixmap( rect.topLeft(), m_customCursor.transformed( transform ) );
            }
        }
        else
        {
            painter->setPen( Qt::black );
            painter->setBrush( Qt::white );
            painter->drawPolygon( m_arrow );
        }

        painter->restore();
        m_previousArrow = m_arrow;

		calcDistanceAzimuth();
    }
    return true;
}
/*
void PositionMarker::update( const ViewportParams* viewport, MapQuality mapQuality )
{
	m_lastBoundingBox = viewport->viewLatLonAltBox();

    if( m_currentPosition != m_previousPosition )
    {
        QPointF screenPosition;

		//see setPosition
		//if( magnitude > 0 && ( qAbs( vector.x() ) - 0.00000001 ) > 0 ) {
		//	m_heading = qAtan( vector.y() / vector.x() ) + M_PI / 2.0;

        viewport->screenCoordinates( m_currentPosition, screenPosition );
		// Calculate the scaled arrow shape
        double rotation = m_heading;
        GeoDataCoordinates top( m_currentPosition.longitude(), m_currentPosition.latitude()+0.1 );
        QPointF screenTop;
        viewport->screenCoordinates( top, screenTop );
        qreal const correction = -90.0 + RAD2DEG * atan2( screenPosition.y()-screenTop.y(), screenPosition.x()-screenTop.x() );
        rotation += correction;

        if ( m_useCustomCursor ) {
            QTransform transform;
            transform.rotate( rotation );
            bool const highQuality = mapQuality == HighQuality || mapQuality == PrintQuality;
            Qt::TransformationMode const mode = highQuality ? Qt::SmoothTransformation : Qt::FastTransformation;
            m_customCursorTransformed = m_customCursor.transformed( transform, mode );
        } else {
            // Calculate the scaled arrow shape
            const QPointF baseX( m_cursorSize, 0.0 );
            const QPointF baseY( 0.0, m_cursorSize );
            const QPointF relativeLeft  = - ( baseX * 9 ) + ( baseY * 9 );
            const QPointF relativeRight =   ( baseX * 9 ) + ( baseY * 9 );
            const QPointF relativeTip   = - ( baseY * 19.0 );
            m_arrow = QPolygonF() << QPointF( 0.0, 0.0 ) << relativeLeft << relativeTip << relativeRight;

            // Rotate the shape according to the current direction and move it to the screen center
            QMatrix transformation;
            transformation.translate( screenPosition.x(), screenPosition.y() );
            transformation.rotate( rotation );
            m_arrow = m_arrow * transformation;

            m_dirtyRegion = QRegion();
            m_dirtyRegion += ( m_arrow.boundingRect().toRect() );
            m_dirtyRegion += ( m_previousArrow.boundingRect().toRect() );
        }

        // Update the trail
        m_trail.push_front( m_currentPosition );
        for( int i = sm_numTrailPoints + 1; i< m_trail.size(); ++i ) {
                m_trail.pop_back();
        
        }
    }
}*/
/*
bool PositionMarker::render( GeoPainter *painter,
                           ViewportParams *viewport,
                           const QString& renderPos,
                           GeoSceneLayer * layer )
{
    Q_UNUSED( renderPos )
    bool const gpsActive = marbleModel()->positionTracking()->positionProviderPlugin() != 0;
	static bool turn=false;
    if ( gpsActive ) {
        m_lastBoundingBox = viewport->viewLatLonAltBox();

    if( m_currentPosition != m_previousPosition ) {
            QPointF screenPosition;
            viewport->screenCoordinates( m_currentPosition, screenPosition );
            const GeoDataCoordinates top( m_currentPosition.longitude(), m_currentPosition.latitude()+0.1 );
            QPointF screenTop;
            viewport->screenCoordinates( top, screenTop );
            qreal const correction = -90.0 + RAD2DEG * atan2( screenPosition.y()-screenTop.y(), screenPosition.x()-screenTop.x() );
            const qreal rotation = m_heading + correction;

            if ( m_useCustomCursor ) {
                QTransform transform;
                transform.rotate( rotation );
                bool const highQuality = painter->mapQuality() == HighQuality || painter->mapQuality() == PrintQuality;
                Qt::TransformationMode const mode = highQuality ? Qt::SmoothTransformation : Qt::FastTransformation;
                m_customCursorTransformed = m_customCursor.transformed( transform, mode );
            } else {
                // Calculate the scaled arrow shape
                const QPointF baseX( m_cursorSize, 0.0 );
                const QPointF baseY( 0.0, m_cursorSize );
                const QPointF relativeLeft  = - ( baseX * 9 ) + ( baseY * 9 );
                const QPointF relativeRight =   ( baseX * 9 ) + ( baseY * 9 );
                const QPointF relativeTip   = - ( baseY * 19.0 );
                m_arrow = QPolygonF() << QPointF( 0.0, 0.0 ) << relativeLeft << relativeTip << relativeRight;

                // Rotate the shape according to the current direction and move it to the screen center
                QMatrix transformation;
                transformation.translate( screenPosition.x(), screenPosition.y() );
                transformation.rotate( rotation );
                m_arrow = m_arrow * transformation;

                m_dirtyRegion = QRegion();
                m_dirtyRegion += ( m_arrow.boundingRect().toRect() );
                m_dirtyRegion += ( m_previousArrow.boundingRect().toRect() );
            }

            // Update the trail
            m_trail.push_front( m_currentPosition );
            for( int i = sm_numTrailPoints + 1; i< m_trail.size(); ++i ) {
                    m_trail.pop_back();
            }
        }

        painter->save();
        painter->autoMapQuality();

        GeoDataAccuracy accuracy = marbleModel()->positionTracking()->accuracy();
        if ( accuracy.horizontal > 0 && accuracy.horizontal < 1000 ) {
            // Paint a circle indicating the position accuracy
            painter->setPen( Qt::transparent );
            int width = qRound( accuracy.horizontal * viewport->radius() / EARTH_RADIUS );
            if ( MarbleGlobal::getInstance()->profiles() & MarbleGlobal::SmallScreen ) {
                int arrowSize = qMax<int>( m_arrow.boundingRect().width(), m_arrow.boundingRect().height() );
                width = qMax<int>( width, arrowSize + 10 );
            }

            painter->setBrush( m_accuracyColor );
            painter->drawEllipse( m_currentPosition, width, width );
        }

        // Draw trail if requested.
        if( m_showTrail ) {
            painter->save();
            // Use selected color to draw trail.
            painter->setBrush( m_trailColor );
            painter->setPen( m_trailColor );
            // we don't draw m_trail[0] which is current position
            for( int i = 1; i < m_trail.size(); ++i ) {
                // Get screen coordinates from coordinates on the map.
				QPointF trailPoint;
                viewport->screenCoordinates( m_trail[i], trailPoint );
       
				const int size = ( sm_numTrailPoints - i ) * 3;
                QRectF trailRect;
                trailRect.setX( trailPoint.x() - size / 2.0 );
                trailRect.setY( trailPoint.y() - size / 2.0 );
                trailRect.setWidth( size );
                trailRect.setHeight( size );

				const qreal opacity = 1.0 - 0.15 * ( i - 1 );
                painter->setOpacity( opacity );
                painter->drawEllipse( trailRect );
               
            }
            painter->restore();
        }

        if( m_useCustomCursor)
        {
            painter->drawPixmap( m_currentPosition, m_customCursorTransformed );
        }
        else
        {
            painter->setPen( Qt::black );
            painter->setBrush( Qt::white );
            painter->drawPolygon( m_arrow );
        }

        painter->restore();
        m_previousArrow = m_arrow;

		calcDistanceAzimuth();

    }
    return true;
}*/

bool PositionMarker::eventFilter(QObject* watched, QEvent* e)
{
    MarbleWidget* marbleWidget = dynamic_cast<MarbleWidget*>( watched );
    Q_ASSERT( marbleWidget );
	if(!marbleWidget)
		return RenderPlugin::eventFilter(watched, e);

	if ( m_marbleWidget==NULL)// != marbleWidget ) 
	{
        // Delayed initialization
        m_marbleWidget = marbleWidget;
		if(m_marbleWidget != NULL )
		 {
			Marble::TreeViewWidget* pTree = m_marbleWidget->tree();
			if(pTree!=NULL)
			{
				connect(this, SIGNAL( signalAddObject( QStringList,long,int,long& ) ),
					(QObject*)(m_marbleWidget->tree()), SLOT( slotAddObject( QStringList,long,int,long& ) ) );
				connect(this, SIGNAL( signalAddPoly(long,int,long,GeoDataLineString&,long&) ),
					(QObject*)(m_marbleWidget->tree()), SLOT( slotAddPoly(long,int,long,GeoDataLineString&,long&) ) );
				connect(this, SIGNAL( signalSetObjectProperties( QString&,long,int) ),
					(QObject*)(m_marbleWidget->tree()), SLOT( slotSetObjectProperties( QString&,long,int) ) );
				connect(this, SIGNAL( signalUpdatePoly(long,int,long,GeoDataLineString&,long&) ),
					(QObject*)(m_marbleWidget->tree()), SLOT( slotUpdatePoly(long,int,long,GeoDataLineString&,long&) ) );

				connect(this, SIGNAL( signalAddImageObject( QString&,long,int,long,long,long,long,long&,bool ) ),
					(QObject*)(m_marbleWidget->tree()), SLOT( slotAddImageObject( QString&,long,int,long,long,long,long,long&,bool ) ) );
				connect(this, SIGNAL( signalAddDescription(QString&,long,long,long,long,long&) ),
					(QObject*)(m_marbleWidget->tree()), SLOT( slotAddDescription(QString&,long,long,long,long,long&) ) );

				connect(this, SIGNAL( signalAddAzimuthLine(long,int,long,double,double,GeoDataLineString&,long&,bool ) ),
					(QObject*)(m_marbleWidget->tree()), SLOT( slotAddAzimuthLine(long,int,long,double,double,GeoDataLineString&,long&,bool ) ) );

			}
		}
	}

	return RenderPlugin::eventFilter(watched, e);
}

QHash<QString,QVariant> PositionMarker::settings() const
{
    QHash<QString, QVariant> settings;

    settings.insert( "useCustomCursor", m_useCustomCursor );
    settings.insert( "cursorPath", m_cursorPath );
    settings.insert( "cursorSize", m_cursorSize );
    settings.insert( "acColor", m_accuracyColor );
    settings.insert( "trailColor", m_trailColor );
    settings.insert( "showTrail", m_showTrail );

    return settings;
}

void PositionMarker::setSettings( const QHash<QString, QVariant> &settings )
{
	const bool smallScreen = MarbleGlobal::getInstance()->profiles() & MarbleGlobal::SmallScreen;
    QColor defaultColor = oxygenBrickRed4;
 
    defaultColor.setAlpha( smallScreen ? 80 : 40 );

	m_useCustomCursor = settings.value( "useCustomCursor", false ).toBool();
    m_cursorPath = settings.value( "cursorPath", m_defaultCursorPath ).toString();
    m_cursorSize = settings.value( "cursorSize", 1.0 ).toFloat();
    loadCustomCursor( m_cursorPath, m_useCustomCursor );

    m_accuracyColor = settings.value( "acColor", defaultColor ).value<QColor>();
    m_trailColor = settings.value( "trailColor", QColor( 0, 0, 255 ) ).value<QColor>();
    m_showTrail = settings.value( "showTrail", false ).toBool();

    readSettings();
}

void PositionMarker::readSettings()
{
    if ( !m_configDialog ) {
        return;
    }

    if( m_useCustomCursor )
        ui_configWidget->m_customCursor->click();
    else
        ui_configWidget->m_originalCursor->click();

    bool found = false;
    float cursorSize = m_cursorSize;
    for( int i = 0; i < sm_numResizeSteps && !found; i++ )
    {
        if( sm_resizeSteps[i] == cursorSize )
        {
            ui_configWidget->m_resizeSlider->setValue( i );
            found = true;
        }
    }
    if( !found )
    {
        ui_configWidget->m_resizeSlider->setValue( sm_defaultSizeStep );
        cursorSize = sm_resizeSteps[sm_defaultSizeStep];
    }

    ui_configWidget->m_sizeLabel->setText( tr( "Cursor Size: %1" ).arg( cursorSize ) );
    QPalette palette = ui_configWidget->m_acColorChooserButton->palette();
    palette.setColor( QPalette::Button,m_accuracyColor );
    ui_configWidget->m_acColorChooserButton->setPalette( palette );
    palette = ui_configWidget->m_trailColorChooserButton->palette();
    palette.setColor( QPalette::Button,m_trailColor );
    ui_configWidget->m_trailColorChooserButton->setPalette( palette );
    ui_configWidget->m_trailCheckBox->setChecked(m_showTrail);
}

void PositionMarker::writeSettings()
{
    if ( !m_configDialog ) {
        return;
    }

	m_useCustomCursor = ui_configWidget->m_customCursor->isChecked();
    m_cursorPath = m_cursorPath;
    m_cursorSize = sm_resizeSteps[ui_configWidget->m_resizeSlider->value()];
    m_accuracyColor = m_accuracyColor;
    m_trailColor = m_trailColor;
    m_showTrail = ui_configWidget->m_trailCheckBox->isChecked();

    emit settingsChanged( nameId() );
}

void PositionMarker::setPosition( const GeoDataCoordinates &position )
{
	//enz
	if(m_lOverlay == 0)
	{
		addOverlay();
	}
		
	if(m_lLayer == 0)
	{
		addLayer();
	}
	if(m_lGroup == 0)
	{
		addGroup(position.longitude(GeoDataCoordinates::Degree),position.latitude(GeoDataCoordinates::Degree) );
	}

	m_heading = marbleModel()->positionTracking()->direction();
	addTrack(position);

    m_previousPosition = m_currentPosition;
    m_currentPosition = position;
	//enz - do i have this value i any position provider plugin?
	

	// Update the trail
/* enz - do i need this ?    
	m_trail.push_front( m_currentPosition );
    for( int i = sm_numTrailPoints + 1; i< m_trail.size(); ++i ) {
            m_trail.pop_back();
    }*/

    if ( m_lastBoundingBox.contains( m_currentPosition ) )
    {
        emit repaintNeeded( m_dirtyRegion );
    }
}
//enz
void PositionMarker::calcDistanceAzimuth()
{
	if(m_previousPosition == m_currentPosition)
		return;

	qreal angle = m_previousPosition.bearing( m_currentPosition, GeoDataCoordinates::Degree, GeoDataCoordinates::FinalBearing );
/*
	qreal dLat = m_currentPosition.latitude()-m_previousPosition.latitude();
	qreal dLon = m_currentPosition.longitude()-m_previousPosition.longitude();
	qreal y = sin(dLon) * cos(m_currentPosition.latitude());
	qreal x = cos(m_previousPosition.latitude())*sin(m_currentPosition.latitude()) -
		sin(m_previousPosition.latitude())*cos(m_currentPosition.latitude())*cos(dLon);
	qreal angle = atan2(y, x)*RAD2DEG;
	*/
	if(m_previousPosition.longitude(GeoDataCoordinates::Degree) > m_currentPosition.longitude(GeoDataCoordinates::Degree))
		angle = 360-angle;
	else angle *= -1;
	
	if( marbleModel()->positionTracking()->direction()!=0)//only turn if driving
		m_marbleWidget->setHeading(angle);

}
/*
//enz
void PositionMarker::calcDistanceAzimuth()
{
	if ( m_gpsTrack == NULL )
		return;

	if(m_gpsTrack->size()<2)
		return;

	GeoDataCoordinates pos1 = m_gpsTrack->at(m_gpsTrack->size()-2);
	GeoDataCoordinates pos2 = m_gpsTrack->at(m_gpsTrack->size()-1);

/*	TCoord  coord1;
	TCoord  coord2;

	coord1.longitude = pos1.longitude(GeoDataCoordinates::Degree);
	coord1.latitude = pos1.latitude(GeoDataCoordinates::Degree);

	coord2.longitude = pos2.longitude(GeoDataCoordinates::Degree);
	coord2.latitude  = pos2.latitude(GeoDataCoordinates::Degree);*/

	/***************************************************************************/
	/*
		var R = 6371; // km
		var dLat = (lat2-lat1).toRad();
		var dLon = (lon2-lon1).toRad();
		var lat1 = lat1.toRad();
		var lat2 = lat2.toRad();

		var a = Math.sin(dLat/2) * Math.sin(dLat/2) +
				Math.sin(dLon/2) * Math.sin(dLon/2) * Math.cos(lat1) * Math.cos(lat2); 
		var c = 2 * Math.atan2(Math.sqrt(a), Math.sqrt(1-a)); 
		var d = R * c;
		*/
/*	qreal dLat = pos2.latitude()-pos1.latitude();
	qreal dLon = pos2.longitude()-pos1.longitude();
	qreal y = sin(dLon) * cos(pos2.latitude());
	qreal x = cos(pos1.latitude())*sin(pos2.latitude()) -
		sin(pos1.latitude())*cos(pos2.latitude())*cos(dLon);
	qreal angle = atan2(y, x)*RAD2DEG;
	/*************************************************************************/
	
//	angle = (qreal)azimuth(&coord1,&coord2);
/*	if(pos1.longitude(GeoDataCoordinates::Degree) > pos2.longitude(GeoDataCoordinates::Degree))
		angle = 360-angle;
	else angle *= -1;
	//qreal qRad = m_marbleWidget->model()->planet()->radius();
	//m_totalDistance = m_tmp_lineString->length(qRad);

	m_marbleWidget->setHeading(angle);

}*/
//enz
void PositionMarker::positionProviderDisabled(bool set)
{
	if(m_lOverlay != 0 && m_lLayer != 0 && m_lGroup != 0)
	{
		if(m_gpsTrack!=NULL)
		{
			m_gpsTrack->clear();
			m_lPoly=0;
		}
		m_lGroup=0;
	}

}

void PositionMarker::chooseCustomCursor()
{
    QString filename = QFileDialog::getOpenFileName( NULL, tr( "Choose Custom Cursor" ) );
    if( !filename.isEmpty() )
        loadCustomCursor( filename, true );
}

void PositionMarker::loadCustomCursor( const QString& filename, bool useCursor )
{
    m_customCursor = QPixmap( filename ).scaled( 22 * m_cursorSize, 22 * m_cursorSize, Qt::KeepAspectRatio, Qt::SmoothTransformation );
    if( !m_customCursor.isNull() )
    {
        if( m_configDialog )
        {
            if( useCursor )
                ui_configWidget->m_customCursor->click();
            ui_configWidget->m_fileChooserButton->setIconSize( QSize( m_customCursor.width(), m_customCursor.height() ) );
            ui_configWidget->m_fileChooserButton->setIcon( QIcon( m_customCursor ) );
        }
        m_cursorPath = filename;
    }
    else
    {
        QMessageBox::warning( NULL, tr( "Position Marker Plugin" ), tr( "Unable to load custom cursor, default cursor will be used. "
                                                       "Make sure this is a valid image file." ), QMessageBox::Ok );
        if ( m_configDialog )
            ui_configWidget->m_fileChooserButton->setIcon( QIcon( m_defaultCursor ) );
        m_customCursor = m_defaultCursor;
        m_cursorPath = m_defaultCursorPath;
    }
}

void PositionMarker::loadDefaultCursor()
{
    m_defaultCursor = QPixmap( m_defaultCursorPath ).scaled( 22 * m_cursorSize, 22 * m_cursorSize, Qt::KeepAspectRatio, Qt::SmoothTransformation );
}

void PositionMarker::chooseColor()
{
    QColor initialColor;
    if( sender() == ui_configWidget->m_acColorChooserButton ) {
        initialColor = m_accuracyColor;
    }
    else if( sender() == ui_configWidget->m_trailColorChooserButton ) {
        initialColor = m_trailColor;
    }
    QColor color = QColorDialog::getColor( initialColor, NULL, 
                                           tr( "Please choose a color" ), 
                                           QColorDialog::ShowAlphaChannel );
    if( color.isValid() )
    {
        QPalette palette;
        if( sender() == ui_configWidget->m_acColorChooserButton ) {
            m_accuracyColor = color;
            palette = ui_configWidget->m_acColorChooserButton->palette();
            palette.setColor( QPalette::Button, m_accuracyColor );
            ui_configWidget->m_acColorChooserButton->setPalette( palette );
        }
        else if( sender() == ui_configWidget->m_trailColorChooserButton ) {
            m_trailColor = color;
            palette = ui_configWidget->m_trailColorChooserButton->palette();
            palette.setColor( QPalette::Button, m_trailColor );
            ui_configWidget->m_trailColorChooserButton->setPalette( palette );
        }
    }
}

void PositionMarker::resizeCursor( int step )
{
    m_cursorSize = sm_resizeSteps[step];
    float newSize = 22 * m_cursorSize;
    m_customCursor = QPixmap( m_cursorPath ).scaled( newSize, newSize, Qt::KeepAspectRatio, Qt::SmoothTransformation );
    ui_configWidget->m_sizeLabel->setText( tr( "Cursor Size: %1" ).arg( m_cursorSize ) );
    if( !m_customCursor.isNull() )
    {
        ui_configWidget->m_fileChooserButton->setIconSize( QSize( m_customCursor.width(), m_customCursor.height() ) );
        ui_configWidget->m_fileChooserButton->setIcon( QIcon( m_customCursor ) );
    }
    loadDefaultCursor();
}

qreal PositionMarker::zValue() const
{
    return 1.0;
}

//enz
void PositionMarker::addOverlay()
{
	long ret = 0;
	if(m_marbleWidget!=NULL)
	{
			QStringList qsl;
			QString qsOver("Position Tracking");
	//		qsOver = QString("Situation-%1").arg(m_iOverlay);
			qsl << "-399" << "-399" << qsOver;

			int iType = 150;//MST_ADD_OVERLAY
			emit signalAddObject(qsl,0,iType,ret);

//			qDebug() << qs  << ret; 
//			m_overlayMap.insert(ret,m_iOverlay);
	
			m_lOverlay = ret;
//			m_iOverlay++;
//			m_addLayer->setEnabled(m_iOverlay>0);

	}
	
}

void PositionMarker::addLayer()
{
	long ret = 0;

	if(m_marbleWidget!=NULL)
	{
			QStringList qsl;
			QString qsLayer("Gps");
			//qsLayer = QString("Layer-%1").arg(m_iLayer);

			qsl << "-399" << "-399" << qsLayer;
			int iType = 151;//MST_ADD_LAYER
			emit signalAddObject(qsl,m_lOverlay,iType,ret);

			QColor qLine(255,0,0);
			QColor qFill(255,0,0,80);

			long lLine = 255;//(long)qLine.rgba();
			lLine <<= 8;
			lLine+= 0;
			lLine <<= 8;
			lLine+= 0;

			long lFill = 130;//(long)qFill.rgba();
			lFill <<= 8;
			lFill+= 192;
			lFill <<= 8;
			lFill+= 255;

			QString qsProp("LINE_COLOR=%1;FILL_COLOR=%2");
			qsProp = qsProp.arg(lLine).arg(lFill);

			iType=161;//MST_SET_LAYER_PROPERTIES
			emit signalSetObjectProperties(qsProp,ret,iType);

			m_lLayer = ret;
//			m_iLayer++;
//			qDebug() << qs  << id << ret; 
	}
}

long PositionMarker::addGroup(qreal lon, qreal lat)
{
	long ret = 0;

	if(m_marbleWidget!=NULL)
	{
			QStringList qsl;
			QString qsLon("%1");
			qsLon = QString("%1").arg(lon);
			QString qsLat("%1");
			qsLat = QString("%1").arg(lat);

			QString qsGroup("Track");
		//	qsGroup = QString("Group-%1").arg(m_iGroup);

			qsl << qsLon << qsLat << qsGroup;
			int iType = 153;//MST_ADD_GROUP
			emit signalAddObject(qsl,m_lLayer,iType,ret);

//			qDebug() << qs  << id << ret; 
	}
	m_lGroup = ret;
//	m_iGroup++;

	return ret;
}

long PositionMarker::addTrack(const GeoDataCoordinates& pos)
{
	int iType = 158;//MST_ADD_POLY
	long ret=0;

	//m_gpsTrack->append(pos);
	*m_gpsTrack << pos;
				
	if(m_gpsTrack->size()>1)
	{
		if(m_lPoly==0)
			emit signalAddPoly(m_lGroup,iType,-1,*m_gpsTrack,ret);
		else
			emit signalUpdatePoly(m_lPoly,iType,-1,*m_gpsTrack,ret);

		m_lPoly=ret;

		if(m_prevHeading!=0)
		{
			long hret=0;
			long hGroup=0;
			//double lon = pos.longitude(GeoDataCoordinates::Degree);
			//double lat = pos.latitude(GeoDataCoordinates::Degree);

			QStringList qsl;
			QString qsLon("%1");
			//qsLon = QString("%1").arg(lon);
			qsLon = QString::number(m_prevLon,'f',8);
			QString qsLat("%1");
			//qsLat = QString("%1").arg(lat);
			qsLat = QString::number(m_prevLat,'f',8);

			QString qsGroup = "TP%1";
			qsGroup = qsGroup.arg(m_gpsTrack->size()-1);

			qsl << qsLon << qsLat << qsGroup;
			int iType = 153;//MST_ADD_GROUP
			emit signalAddObject(qsl,m_lLayer,iType,hGroup);

			QString qsImg=MarbleDirs::path( "bitmaps/coordinate.png" );

			//QString qsHeading="%1;%2";
			//qsHeading = qsHeading.arg(m_heading).arg(qsImg);
			iType = 154;//MST_ADD_ICON
			emit signalAddImageObject(qsImg,hGroup,iType,0,0,0,-1,hret,false);

			char deg='°';
		/*	qreal speed =  marbleModel()->positionTracking()->speed();
			qreal qRad = marbleModel()->planet()->radius();
			qreal distance = m_gpsTrack->length(qRad)/1000.0;*/

			qsImg.clear();
			qsImg = "Heading: %1 %2;Speed:    %3 km/h;Distance: %4 km"; 
			qsImg = qsImg.arg(m_prevHeading).arg(deg).arg(m_prevSpeed).arg(m_prevDistance);
			emit signalAddDescription(qsImg,hGroup,111,1,0,hret);

	/*		GeoDataLineString gdl;
			long azi=(long)m_prevHeading;
			iType = 155;//MST_ADD_AZIMUTH
			emit signalAddAzimuthLine(hGroup,iType,azi,(double)m_prevLon,(double)m_prevLat,gdl,hret,false);

			calcAzimuthLine((double)m_prevLon,(double)m_prevLat,azi,&gdl);
			iType = 181;//MST_MOVE_AZIMUTH
			hGroup=0;
			emit signalAddAzimuthLine(hret,iType,azi,(double)m_prevLon,(double)m_prevLat,gdl,hGroup,false);
*/
			//remember for next placemark setting
			m_prevHeading   = m_heading;
			m_prevLon       = pos.longitude(GeoDataCoordinates::Degree);
			m_prevLat       = pos.latitude(GeoDataCoordinates::Degree);
			m_prevSpeed     = marbleModel()->positionTracking()->speed();
			qreal qRad      = marbleModel()->planet()->radius();
			m_prevDistance  = m_gpsTrack->length(qRad)/1000.0;

		}
		else
		{
			//remember for next placemark setting - we allways set the last-1 placemark
			m_prevHeading   = m_heading;
			m_prevLon       = pos.longitude(GeoDataCoordinates::Degree);
			m_prevLat       = pos.latitude(GeoDataCoordinates::Degree);
			m_prevSpeed     = marbleModel()->positionTracking()->speed();
			qreal qRad      = marbleModel()->planet()->radius();
			m_prevDistance  = m_gpsTrack->length(qRad)/1000.0;
		}
	}
	else
	{
		//remember for next placemark setting - we allways set the last-1 placemark
		m_prevHeading   = m_heading;
		m_prevLon       = pos.longitude(GeoDataCoordinates::Degree);
		m_prevLat       = pos.latitude(GeoDataCoordinates::Degree);
		m_prevSpeed     = marbleModel()->positionTracking()->speed();
		qreal qRad      = marbleModel()->planet()->radius();
		m_prevDistance  = m_gpsTrack->length(qRad)/1000.0;
	}

	return ret;
}


}

Q_EXPORT_PLUGIN2( PositionMarker, Marble::PositionMarker )

#include "moc_PositionMarker.cpp"
