/******************************************************************************************************************
 *	@file			MapViewLayer.h
 *	@copyright		2010 Rohde&Schwarz, Munich
 *	@version		$Header: MapViewLayer.h    
 *	5 Oktober 2011  14:49:35  enzensbe $
 *
 *	@author			Konrad Enzensberger		e.konrad@mpegcode.com
 *	@responsible	KOnrad Enzensberger
 * 
 *	@language		C++ ANSI V3
 *
 *	@description 	This file defines the MapView geo object layer.
 *					
 *					
 *****************************************************************************************************************/

#ifndef MARBLE_MAPVIEWLAYER_H
#define MARBLE_MAPVIEWLAYER_H

#include "GeoDataCoordinates.h"
#include "LayerInterface.h"

#include <QtCore/QModelIndex>
#include <QtGui/QItemSelection>
#include <QtGui/QMenu>

#include "Enum.h"
//#include "treeviewitem.h"
using namespace GIS;
//using namespace Model;

class QAbstractProxyModel;
//class QComboBox;

#define PI		3.14159265358979323846

namespace Marble
{
	static const qreal s_labelOutlineWidth = 2.5;

class MarbleWidget;
class TreeViewModel;
class MarblePlacemarkModel;
class MapViewLayerPrivate;
class GeoDataDocument;
class GeoDataFolder;
//class RouteSkeleton;
//class AlternativeRoutesModel;

/**
  * @brief A paint layer that serves as a view on a route model
  */
class MapViewLayer: public QObject, public LayerInterface
{
    Q_OBJECT

public:
    /**
      * @brief Constructor
      * @param widget The marble widget used for geopos <-> screenpos transformations
      *   and repainting of (small) areas. Must not be null
      * @param parent Optional parent widget
      */
    explicit MapViewLayer( MarbleWidget *widget, QWidget *parent = 0, qint64 index=0,long interfaceID=0);

    /** Destructor */
    ~MapViewLayer();

    /** Reimplemented from LayerInterface. We'll hover above the surface */
    QStringList renderPosition() const;

    /** Reimplemented from LayerInterface. Paints route items and placemarks */
    bool render( GeoPainter *painter, ViewportParams *viewport,
                 const QString &renderPos = "NONE", GeoSceneLayer *layer = 0 );

    /**
      * Set the proxy model another QAbstractItemView uses that should share
      * its selection model with us. Needed because this class uses an unfiltered
      * model which has different indices than a filtered one.
      */
    void synchronizeWith( QItemSelectionModel *selection );

	/**
      * Tree selection forwarded to synchronize with layer item's
      */
    void synchronizeWithTree( const QModelIndex& idx);

	/**
      * Extern selection forwarded to synchronize with layer and tree item's
      */
    void synchronizeWithExtern( const QModelIndex& idx,bool ext);

    /**
      * Set the routing model to use. Implicitly removes the placemark model.
      */
    void setModel ( TreeViewModel *model );

    /**
      * Set the placemark model to use. Implicitly removes the routing model.
      */
    void setModel ( MarblePlacemarkModel *model );

    /**
      * When enabled, a cross hair cursor is shown in the map and each click
      * in the map is reported via the pointSelected signal. Mouse move,
      * press and release events are fully eaten to avoid things like context
      * menus chiming in.
      */
    void setPointSelectionEnabled( bool enabled );

    /**
      * The route is dirty (needs an update) and should be painted to indicate that
      */
    void setLayerDirty( bool dirty );

    /**
      * get the Interface ID for TcpIp writing
      */
	long getInterfaceID(void){return m_InterfaceID;}

	 /**
      * set the layer line color 
      */
	void setLineColor(long col);
	 /**
      * set the layer line color 
      */
	void setLineColor(QColor&);
	 /**
      * get the layer line color 
      */
	QColor& getLineColor();
	/**
      * set the layer fill color 
      */
	void setFillColor(long col);
	/**
      * set the layer fill color 
      */
	void setFillColor(QColor&);
	/**
      * get the layer fill color 
      */
	QColor& getFillColor();
	/**
      * set the layer font color 
      */
	void setFontColor(long col);
	/**
      * get the layer font color 
      */
	QColor& getFontColor();
	/**
      * set the layer font back color 
      */
	void setFontBackColor(long col);
	/**
      * get the layer font back color 
      */
	QColor& getFontBackColor();
	/**
      * set the layer font
      */
	void setDefaultFont( const QFont& font );
	/**
      * set the layer transparency 
      */
	void setTransparency(int trans);
	/**
      * get the layer transparency 
      */
	int getTransparency();
	/**
      * set the layer line width 
      */
	void setLineWidth(int width);
	/**
      * get the layer line width 
      */
	int getLineWidth();
	/**
      * set the marker size 
      */
	void setMarkerSize(int size);
	/**
      * set the layer editable 
      */
	void setEdit(bool edit);
	/**
      * get the layer editable flag
      */
	bool getEdit(void);
	/**
      * set the layer deleteable 
      */
	void setDelete(bool del);
	/**
      * get the layer deleteable flag
      */
	bool getDelete(void);
	/**
      * set the layer objects delete able 
      */
	void setObjectDelete(bool del);
	/**
      * get the layer objects delete able flag 
      */
	bool getObjectDelete(void);
	/**
      * set the layer objects moveable 
      */
	void setObjectMove(bool move);
	/**
      * set the layer objects moveable/deleteable 
      */
	void setObjectEdit(bool move);
	/**
      * set the layer context menu 
      */
	void setContextMenu(QString qsMenu);

	/**
      * export layer objects to kml
      */
	void export2kml(QString&);

	/**
      * export overlay objects to kml
      */
	void exportOverlay2kml(GeoDataDocument* document,GeoDataFolder* folder);

	/**
      * export group objects to kml
      */
	void exportGroup2kml(GeoDataDocument* document,GeoDataFolder* folder,long groupID,long objectID);


	/**
      * set the layer zvalue
      */
	void setLayerZValue(qreal zValue);

	/**
	*  Overriding LayerInterface to paint on top of other layers
	*/
    virtual qreal zValue() const;

	/**
      * set dimmer value to change text color
      */
	void setDimmerValue(int dValue);

	/**
      * set pen style to change pen style
      */
	void setPenStyleValue(Qt::PenStyle style);

	/**
      * get pen styly
      */
	Qt::PenStyle getPenStyleValue();

	/**
      * enable region calculation for layer objects
      */
	void setLayerUpdate(bool val);

	/**
      * input enabled flag - if true, reject mouse move over object
      */
	void setInputEnabled(bool val);

	/**
      * input enabled flag from float items - if true, reject mouse move over object
      */
	void setInputTempEnabled(bool val);

	/**
      * set the layer as a legend layer - not geo projected
      */
	void setLegendLayer(bool legend);

	/**
      * get legend layer state
      */
	bool getLegendLayer();

	/**
      * set the layer legend alignment - not geo projected
      */
	void setLegendLayerAlignment(int align);

	/**
      * set the layer legend rect
      */
	void setLegendRect(QRect& rect);

	/**
	  * get all intersected objects
    */
	int intersectionObjects(QModelIndex& idx,QPoint pt,bool flag,int pos);

	/**
	  * get all intersected objects from rubber band region
    */
	int intersectionRegionObjects(QModelIndex& idx,const QRect&,bool flag,int pos);

	/**
	*	used to display layername in runtime trace 
	*/
	virtual QString runtimeTrace() const;

	/**
      * Set the view context to determine whether the map is used interactively
      */
    void setViewContext( ViewContext viewContext );


Q_SIGNALS:
    /**
      * A placemark was selected (clicked) by the user. The index belongs to
      * the model set via setModel
      */
    void placemarkSelected( const QModelIndex &index,long intID,bool sel );

	/**
      * A placemark was selected (clicked) by the user. The index belongs to
      * the model set via setModel
      */
    void placemarkMultiSelected( const QModelIndex &index,long intID,bool sel );

	 /**
      * A placemark was double clicked
      */
    void placemarkDblClick( const QModelIndex &index,long intID,QPoint& pt,bool sel );

	 /**
      * A polygone was selected (clicked) by the user. The index belongs to
      * the model set via setModel
      */
    void polygoneSelected( const QModelIndex &index,long intID,bool sel );

	 /**
      * A polygone was double clicked
      */
    void polygoneDblClick( const QModelIndex &index,long intID,QPoint& pt,bool sel );

	 /**
      * A placemark was moved by the user. The index belongs to
      * the model set via setModel
      */
    void placemarkMoved( const QModelIndex &index,long intID );

	 /**
      * A polygone was moved by the user. The index belongs to
      * the model set via setModel
      */
    void polygoneMoved( const QModelIndex &index,long intID );

    /**
      * A point was selected by a mouse click after setPointSelectionEnabled(true) was called
      */
    void pointSelected( const GeoDataCoordinates &coordinates );

    /**
      * Selection of points was aborted by the user without selecting a point
      */
    void pointSelectionAborted();

	 /** User deleted an object */
    void deleteItem ( const QModelIndex &index,long intID );

    /**
      * Route should be exported
      */
    //void exportRequested();

	 /**
      * Selection of an object context menu by the user
      */
	void contextMenuSelected( const QModelIndex &index,long intID, int& iVal );

	/**
      * mouse hovers object signal - used for send corresponding event
      */
	void mouseHoversObject( const QModelIndex &index,QPoint& pos,long intID );

	/**
      * send mouse pos if mouse hovers object
      */
	void mouseMoveGeoPosition( QString );

	/**
      * A placemark was selected (clicked) by the user. The index belongs to
      * the model set via setModel
      */
    void placemarkMultiSelection( const QMap<int,QModelIndex> &idxMap,long intID,bool sel );

public Q_SLOTS:
	/**
      * free selection index if item is deleted by tree
      */
	void slotDeleteTreeItem(int,long);

protected:
    /** Overriding QWidget, used to make the layer interactive */
    bool eventFilter( QObject *obj, QEvent *event );

private Q_SLOTS:
    void removeViaPoint();
	void insertViaPoint();
	void deleteObject();

	 /**
      * The viewport has changed, recalculate positions accordingly
      */
    void setViewportChanged();

	/**
      * send layer context menu id to TcpIp
	  * called from: 
	  * - layer context menu
	  * - multi select context menu 
      */
	void slotLayerContextMenu(QAction*);

	/**
      * set item id for multi select context menu (only used if layer context menu is set)
      */
	void slotLayerContextMenuHovered(QAction*);

	/**
	  * set item selection in tree, send selection to TcpIp
      * called by "select" menu, from multi select context menu (added to layer context menu)
      */
	void slotSelectContextMenu();

	 /**
	  * set item selection in tree, send selection to TcpIp
      * called by multi select context menu (no layer context menu is set)
      */
	void slotSelectActionMenu(QAction*);

private:
    MapViewLayerPrivate *const d;
    friend class MapViewLayerPrivate;
	long	m_InterfaceID;
	qreal	m_zValue;
	int		m_cmID;

};

} // namespace Marble

#endif
